package org.objectweb.orchestra.samples.BSOASender.bf;

import javax.jms.ConnectionFactory;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.MessageProducer;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;

public class JmsBSOASender {

  final boolean debug = Boolean.getBoolean("org.objectweb.orchestra.samples.BSOASender.JmsBSOASender.debug");

  final static String appUri = "urn:org.objectweb.orchestra.samples.BSOASender.svc";
  final static String appPfx = "a";

  final static String soapUri = "http://schemas.xmlsoap.org/soap/envelope/";
  final static String soapPfx = "s";
  final static String soapNs = "xmlns:" + soapPfx + "=\"" + soapUri + "\"";

  /**
   * Sends a message to a JMS Queue
   * @param queue name of queue
   * @param message message to be sent
   */
  void send(String queueName, String message) throws Exception {
    Context jndi = new InitialContext();
    QueueConnectionFactory  qcf = (QueueConnectionFactory)jndi.lookup("QCF");
    Queue queue=(Queue) jndi.lookup(queueName);
    QueueConnection qc = qcf.createQueueConnection();
    Session session = qc.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
    MessageProducer sender = session.createProducer(queue);
    TextMessage tm = session.createTextMessage(message);
    sender.send(tm);
    session.close();
    qc.close();
  }


  /**
   * Send a message to a JMS Queue.
   *
   * @param soapEnvelope
   *          Soap request containing parameters to be assigned to an input
   *          record.
   * @param svc
   *          is not used by the echo sender.
   * @return a String containing a Soap response envelope
   * @throws Exception
   */
  public String invoke(String soapEnvelope, String svc) throws Exception {
    if (debug) System.out.println(soapEnvelope);

    SoapParser parser = new SoapParser(soapEnvelope);
    parser.addNamespace(soapPfx, soapUri);
    parser.addNamespace(appPfx, appUri);
    String queue = parser.getNodeValue("//a:send/a:queue");
    String message = parser.getNodeValue("//a:send/a:message");
    String sendReturn = "OK"; // default result message


    /*
     * send the message to the JMS Queue
     */
    try {
      send(queue, message);
    } catch (Exception e) {
      sendReturn = e.toString();
    }

    /*
     * set up the soap response envelope
     */
    StringBuffer sb = new StringBuffer();
    sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
            + "<s:Envelope " + soapNs + "  >\n"
            + "<s:Body>\n"
            + "<sendResponse xmlns=\"urn:org.objectweb.orchestra.samples.BSOASender.svc\">\n"
            + "    <sendReturn>"
            +      sendReturn
            +      "</sendReturn>\n"
            + "</sendResponse>\n"
            + "</s:Body>\n"
            + "</s:Envelope>\n");

    String result = sb.toString();

    if (debug)
      System.out.println(result);
    return result;
  }

  public static void main(String args[]) throws Exception {

    StringBuffer sb = new StringBuffer();

    sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
      "<s:Envelope " + soapNs + ">\n" +
      "<s:Body>\n" +
      "<print xmlns=\"urn:org.objectweb.orchestra.samples.BSOASender.svc\">\n" +
      "<message>");

    if (args.length > 0) {
      for (int i=0; i<args.length; ++i) {
        sb.append(args[i]).append(" ");
      }
    }
    else {
      sb.append("main: default message");
    }

    sb.append("</message>\n" +
      "</print>\n" +
      "</s:Body>\n" +
      "</s:Envelope>\n");
    ConsoleBSOASender sender = new ConsoleBSOASender();
    sender.invoke(sb.toString(), null);
  }
}