package org.objectweb.orchestra.samples.BSOASender.bf;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.xml.serializer.Serializer;
import org.apache.xml.serializer.DOMSerializer;
import org.apache.xml.serializer.SerializerFactory;
import org.apache.xml.serializer.OutputPropertiesFactory;
import org.apache.xml.serializer.Method;

import org.jaxen.JaxenException;
import org.jaxen.XPath;
import org.jaxen.SimpleNamespaceContext;

import org.w3c.dom.CharacterData;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

class SoapParser {
  final Node doc;
  final SimpleNamespaceContext nsCtx = new SimpleNamespaceContext();

  final static String soapUri = "http://schemas.xmlsoap.org/soap/envelope/";

  /**
   * Constructs SoapParser from a String containing a soap envelope
   * @param soapEnvelope
   */
  SoapParser(String soapEnvelope) {

    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    factory.setNamespaceAware(true);
    Document doc = null;
    try {
      DocumentBuilder builder = factory.newDocumentBuilder();
      InputSource is = new InputSource(new StringReader(soapEnvelope));
      doc = builder.parse(is);
    } catch (SAXException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    } catch (IOException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    } catch (ParserConfigurationException e1) {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }

    this.doc = doc.getDocumentElement();
 }

  Node getNode(String xpath) throws JaxenException {
    XPath expression = new org.jaxen.dom.DOMXPath(xpath);
    expression.setNamespaceContext(nsCtx);
    return (Node) expression.selectSingleNode(doc);
  }

  String getNodeValue(String xpath) {
    try {
      Node result = getNode(xpath);
      if (result != null) {
        CharacterData cd = (CharacterData) result.getFirstChild();
        return (cd == null) ? "" : cd.getData();
      }
    } catch (JaxenException e) {
      e.printStackTrace();
    }
    return null;
  }

  /**
   * returns a fragment of the xml document starting at the
   * node defined by xpath.
   * <p>For example, this method can be used to retrieve
   * the content of the soap body as a String of XML.
   * @param xpath specifies the first node to return.
   * @return String containing the requested fragment of xml.
   */
  String getNodeXml(String xpath) {
    try {
      Node result = getNode(xpath);
      if (result != null) {
        Properties pSer = OutputPropertiesFactory.getDefaultMethodProperties(Method.XML);
        pSer.setProperty("omit-xml-declaration", "yes");
        Serializer ser = SerializerFactory.getSerializer(pSer);
        ser.setWriter(new StringWriter());
        ser.asDOMSerializer().serialize(result);
        return ser.getWriter().toString();
      }

    } catch (JaxenException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
    return null;
  }

  String getNodeName(String xpath) {
    try {
      Node result = getNode(xpath);
      if (result != null) {
        return result.getNodeName();
      }
    } catch (JaxenException e) {
      e.printStackTrace();
    }
    return null;
  }

  /**
   * Called by application prior to invoking any of the getXxx methods.
   * <p>Application specifies the prefix and associated uri for each
   * namespace that will be used in an xpath expression.
   *
   * @param pfx The namespace prefix to be used within xpath expressions.
   * <p>The xpath namespace prefix is completely distinct from any prefix
   * that is used within the document.  The document might use a prefix
   * of "soap" or "soapenv" while the xpath uses simply "s".
   * @param uri The URI associated with a prefix.  The URI must
   * match exactly the URI specified in the document.
   */
  void addNamespace(String pfx, String uri) {
    nsCtx.addNamespace(pfx, uri);
  }

}