package org.objectweb.orchestra.samples.weather2;

import java.lang.*;
import java.math.*;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;


public class Weather2Client
{
    public static void main(String [] args)
    {
        String location = getLocation(args);

        if (location.length() > 0) {
            try {
//              location = "1600 Pennsylvania Avenue, Washington DC";
                System.out.println("Location: "+ location);

                WeatherServiceBP is=new WeatherServiceBPLocator();
                WeatherPT ipt=is.getweatherPT();

                LocationType in = new LocationType();
                in.setLocation(location);
                //
                // Put current date in
                //
                Date d = new Date();
                Calendar cal = new GregorianCalendar();
                cal.setTime(d);
//              cal.add(Calendar.SECOND,7);
                in.setDate(cal);


                OutputType out = ipt.process(in);

                System.out.println("Result = " + out.getResult());

            } catch (NumberFormatException e) {
                System.out.println(">>>>> ERROR: check input params...");
            } catch (Exception e) {
                e.printStackTrace(System.err);
            }
        } else {
            displayUsage();
        }
    }

    private static String getLocation(String [] args) {
        boolean invalidInput;

        //
        // We only want 1 arg
        //
        if (args.length != 1) {
            return "";
        }
        //
        // It needs to be at least 1 character
        //
        String loc = args[0];
        if (loc.length() == 0) {
            return "";
        }
        //
        // If more than 1 character, then assume it is a complete address
        //
        if (loc.length() > 1) {
            if (loc.length() < 10) {
                return "";
            }
            return loc;
        }

        try {
            int i = new Integer(loc).intValue();
            switch (i) {
            case 1:
                return "1600 Pennsylvania Avenue, Washington DC";

            case 2:
                return "601 Whitehead St, Key West, Fl";

            case 3:
                return "13430 N Black Canyon Highway, Phoenix Az";

            case 4:
                return "400 Broad St, Seattle Wa";

            case 5:
                return "233 Wacker Drive, Chicago, Il";

            case 6:
                return "350 Fifth Ave, New York NY";

            default:
                return "";
            }
        } catch (Exception e) {
            //
            // Non numeric single character input can't be an address
            //
            return "";
        }
    }

    private static void displayUsage() {
        System.out.println(">>>>> ERROR: Enter a single integer value to select an address from the following list:");
        System.out.println(">>>>>         1 = 1600 Pennsylvania Avenue, Washington DC");
        System.out.println(">>>>>         2 = 601 Whitehead St, Key West, Fl");
        System.out.println(">>>>>         3 = 13430 N Black Canyon Highway, Phoenix Az");
        System.out.println(">>>>>         4 = 400 Broad St, Seattle Wa");
        System.out.println(">>>>>         5 = 233 Wacker Drive, Chicago, Il");
        System.out.println(">>>>>         6 = 350 Fifth Ave, New York NY");
        System.out.println(">>>>> ");
        System.out.println(">>>>>         Or enter a complete US address within single quotes.");
    }
}
