/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tests.jbi;

import java.io.File;
import java.io.StringReader;
import java.net.URL;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import javax.xml.transform.dom.DOMSource;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.Util;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.XMLFilterImpl;


public class JbiJunitTestCase extends TestCase {

    protected String targetURL = "http://localhost:8192/junit_HttpEndpoint_";
    protected String messageChildDir = "messages";
    protected String inMessageName = "inMess";
    protected String outMessageName = "outMess";
    protected String inMessagePath = null;
    protected String processName = null;
    protected String sep = File.separator;
    protected String extension = ".xml";

    public JbiJunitTestCase(String testname) {
        super(testname);
    }

    public static TestSuite suite() {
        return new TestSuite(JbiJunitTestCase.class);
    }

    public void setUp() throws Exception {
        //nothing to do
    }

    public void callAndVerify(String inMessageNb, String outMessageNb) {
        Document response = call(inMessageNb, outMessageNb);
        verify(response, inMessageNb, outMessageNb);
    }

    public Document call(String inMessageNb, String outMessageNb) {
        Document response = null;
        try {
            this.inMessagePath = getInMessagePath();
            String toPrint = this.processName+", inMessageNb="+inMessageNb+", outMessageNb="+outMessageNb;
            System.out.println(BPELConstants.TEST_MESS_BEGIN+toPrint+BPELConstants.TEST_MESS_END);
            SOAPMessage returnMess = getCnx().call(getInMessage(inMessageNb),getURL());
            response = getDocumentFromMessage(returnMess);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return response;
    }

    public void verify(Document response, String inMessageNb, String outMessageNb) {
        String responseString = null;
        boolean result = false;
        try {
            String res = compare(response,outMessageNb);
            result = res.equals("OK");
            if (!result) {
                responseString = Util.getNodeAsString(response);
                System.out.println("compare, processName="+this.processName+", inMessageNb="+inMessageNb+", outMessageNb="+outMessageNb+", cause="+res);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        assertTrue("Error : processName="+this.processName+", inMessageNb="+inMessageNb+", outMessageNb="+outMessageNb+", return=\n"+responseString,result);

    }

    private String getInMessagePath() {
        String basePack = System.getProperty("orchestra.jbi.junit.package");
        String myPackageName = this.getClass().getPackage().getName();
        String currentDir = myPackageName.substring(basePack.length()+1);
        currentDir = currentDir.replace('.',File.separatorChar);
        return System.getProperty("orchestra.jbi.junit.basedir")+this.sep+currentDir;
    }

    private Document getDocumentFromMessage(SOAPMessage mess) throws Exception {
        SOAPBody body = mess.getSOAPBody();
        int size = body.getChildNodes().getLength();
        if (size != 1) {
            throw new Exception("There are "+size+" children in the returned SOAPBody. Only one was expected");
        }
        Node n = (Node) body.getChildElements().next();
        //InputStream is = new StringReader(n.toString());
        //this works as n is an axis node having a toString method...
        InputSource is = new InputSource(new StringReader(n.toString()));
        //InputStream is = new ByteArrayInputStream(Util.getNodeAsString(n).getBytes());
        Document doc = getBuilder().parse(is);
        //System.out.println("getDocumentFromMessage, doc="+Util.getNodeAsString(n));
        return doc;
    }

    private DocumentBuilder getBuilder() throws Exception {
        DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
        dbFactory.setValidating(true);
        dbFactory.setNamespaceAware(true);
        dbFactory.setIgnoringElementContentWhitespace(true);
        DocumentBuilder builder = dbFactory.newDocumentBuilder();
        ErrorHandler eh = new XMLFilterImpl();
        builder.setErrorHandler(eh);
        return builder;
    }

    private SOAPConnection getCnx() throws Exception {
        SOAPConnectionFactory conf = SOAPConnectionFactory.newInstance();
        SOAPConnection con = conf.createConnection();
        return con;
    }

    private SOAPMessage getInMessage(String inMessageNb) throws Exception {
        //get a DOMSource from the XML content
        Document document = getBuilder().parse(getPath("in", inMessageNb));
        DOMSource domSource = new DOMSource(document);
        return getMessage(domSource);
    }

    private SOAPMessage getMessage(DOMSource ds) throws Exception {
        //creates the SOAPMessage and add the SOAPPart containing the body
        MessageFactory mf = MessageFactory.newInstance();
        SOAPMessage sm = mf.createMessage();
        SOAPPart soapPart = sm.getSOAPPart();
        soapPart.setContent(ds);
        return sm;
    }

    private URL getURL() throws Exception {
        return new URL(this.targetURL + this.processName);
    }

    private String getPath(String mode, String nb) {
        String name = null;
        if (mode.equals("in")) {
            name = this.inMessageName;
        } else {
            name = this.outMessageName;
        }
        return this.inMessagePath+this.sep+this.messageChildDir+this.sep+name+nb+this.extension;
    }

    private String compare(Document response, String outMessageNb) throws Exception {
        Document expected = getBuilder().parse(getPath("out",outMessageNb));
        Element expectedEl = expected.getDocumentElement();
        Element responseEl = response.getDocumentElement();
        String res = nodeEquals(expectedEl,responseEl);
        if (!res.equals("OK")) {
            System.out.println("Expected = \n"+Util.getNodeAsString(expectedEl));
            System.out.println("Response = \n"+Util.getNodeAsString(responseEl));
        }
        return res;
    }

    private String nodeEquals(Node n1, Node n2) {
        try {
            n1.normalize();
            n2.normalize();
            //System.out.println("\nn1="+Util.getNodeAsString(n1));
            //System.out.println("\nn2="+Util.getNodeAsString(n2));
            if (n1==null && n2==null) {
                return "OK";
            } else if ((n1==null && n2!=null) || (n1!=null && n2==null)) {
                return "n1="+n1+", n2="+n2;
            }
            if (!areEquals(n1.getNamespaceURI(),n2.getNamespaceURI())) {
                return "n1.namespaceURI="+n1.getNamespaceURI()+", n2.namespaceURI="+n2.getNamespaceURI();
            }
            if (!areEquals(n1.getLocalName(),n2.getLocalName())) {
                return "n1.localName="+n1.getLocalName()+", n2.localName="+n2.getLocalName();
            }
            if (n1.getNodeType()!=n2.getNodeType()) {
                return "n1.nodeType="+n1.getNodeType()+", n2.nodeType="+n2.getNodeType();
            }
            if (!areEquals(n1.getNodeValue(),n2.getNodeValue())) {
                return "n1.nodeValue='"+n1.getNodeValue()+"'\nn2.nodeValue='"+n2.getNodeValue()+"'";
            }
            /*
            if (!areEquals(n1.getTextContent(),n2.getTextContent())) {
                return "6, n1.tc="+n1.getTextContent()+", n2.tc="+n2.getTextContent();
            }
             */
            ArrayList n1Children = getElementNodes(n1);
            ArrayList n2Children = getElementNodes(n2);
            if (n1Children.size() != n2Children.size()) {
                //System.out.println("\nn1="+Util.getNodeAsString(n1));
                //System.out.println("\nn2="+Util.getNodeAsString(n2));
                return "n1.childrenNb="+n1Children.size()+", n2.childrenNb="+n2Children.size();
            }
            for (int i=0;i<n1Children.size();i++) {
                Node n1Child = (Node) n1Children.get(i);
                Node n2Child = (Node) n2Children.get(i);
                String st = nodeEquals(n1Child,n2Child);
                if (!st.equals("OK")) {
                    return st;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "OK";
    }

    private ArrayList getElementNodes(Node n) {
        ArrayList list = new ArrayList();
        ArrayList texts = new ArrayList();
        NodeList children = n.getChildNodes();
        for (int i=0;i<children.getLength();i++) {
            Node child = children.item(i);
            if (child.getNodeType()==Node.ELEMENT_NODE) {
                list.add(child);
            } else if (child.getNodeType()==Node.TEXT_NODE) {
                texts.add(child);
            }
        }
        if (list.size()==0) {
            list.addAll(texts);
        }
        return list;
    }

    private boolean areEquals(String s1,String s2) {
        if (s1==null & s2==null) {
            return true;
        } else if (s1==null && s2!=null) {
            return false;
        } else if (s1 !=null && s2==null) {
            return false;
        }
        return s1.equals(s2);
    }

}
