package org.objectweb.orchestra.tests.load;

import java.util.Date;
import java.util.HashMap;
import java.util.Vector;
import java.io.File;
import java.io.FileWriter;

import java.lang.reflect.Constructor;

import org.objectweb.orchestra.tests.load.LoadAbstractThread;
import org.objectweb.orchestra.tests.load.LongElement;
import org.objectweb.orchestra.tests.load.LongThread;
import org.objectweb.orchestra.util.BPELString;

public class LoadClient
{
    //CONFIGURATION
    private static int testNb = -1;
    private static String mode = null;
    private static long time_between_processes = -1;
    private static long timeBetweenPrints = -1;
    private static long loadTime = -1;
    private static Vector toLaunch = null;
    private static String output = null;
    private static boolean exitOnError = false;
    private static Vector longList = null;
    private static int longCpt = 0;
    private static int maxAllowedThreads = -1;
    private static long time_to_load = -1;
    private static int axisTimeOut = 0;
    private static int currentProcessNb = 0;

    private static int instancesNb = 0;
    private static int mesuredNb = 0;
    private static int notMesuredNb = 0;
    private static int successNb = 0;
    private static int errorsNb = 0;
    private static int currentLongThreads = 0;
    private static int finishedLongThreads = 0;
    private static boolean tooManyThreads = false;
    private static long totalExecTime = 0;

    private static String toLaunchString = null;
    private static FileWriter fw = null;
    private static FileWriter ew = null;
    private static Vector processes = null;
    private static long startTime = -1;
    private static long startMesuringTime = -1;
    private static long endTime = -1;
    private static long lastPrint = 0;
    private static File file = null;
    private static String longListSt = null;

    private static HashMap bestTimes = null;
    private static HashMap worthTimes = null;
    private static boolean counting = false;

    private static Object mutex = null;

    public static void main(String [] args) {
        mutex = new Object();
        bestTimes = new HashMap();
        worthTimes = new HashMap();
        try {
            analyseArgs(args);
            initTimes();
            createLogFile();
            createErrorsFile();
            //move from minutes to miliseconds
            loadTime = loadTime*60000;
            time_to_load = time_to_load *1000;
            startTime = new Date().getTime();
            lastPrint = startTime;

            if (mode.equals("stress")) {
                startMesuringTime = startTime + time_to_load;
            } else {
                startMesuringTime = startTime;
            }

            if (mode.equals("long")) {
                launchNextLongThread();
            } else {
                Vector initialThreads = new Vector();
                synchronized (mutex) {
                    for (int i=0;i<testNb;i++) {
                        initialThreads.addElement(buildThread(instancesNb++));
                    }
                    for (int p = 0; p < initialThreads.size() ; p++) {
                        ((Thread)initialThreads.get(p)).start();
                        Thread.sleep(time_between_processes);
                    }
                    Date end = new Date(startMesuringTime+(loadTime));
                    endTime = end.getTime();
                }
            }
        } catch (Exception e) {
            System.out.println("ERROR: Problem initializing LoadClient :\n"+e);
            e.printStackTrace();
        }
    }

    private static void initTimes() {
        for (int i=0;i<processes.size();i++) {
            bestTimes.put(processes.get(i),new Long(100000000));
            worthTimes.put(processes.get(i),new Long(0));
        }

    }

    private static void updateTimes(String processName,long execTime) {
        Long exec = new Long(execTime);
        long best = ((Long)bestTimes.get(processName)).longValue();
        if (execTime < best) {
            bestTimes.put(processName,exec);
        }
        long worth = ((Long)worthTimes.get(processName)).longValue();
        if (execTime > worth) {
            worthTimes.put(processName,exec);
        }
    }

    public static void endWait (LoadAbstractThread t) {
        try {
            synchronized (mutex) {
                long currentTime = new Date().getTime();
                boolean canFinish = false;
                if (!counting) {
                    counting = currentTime >= startMesuringTime;
                    if (counting) System.out.println("Starting the mesures");
                }
                if (counting) {
                    if (t.success) {
                        successNb++;
                    } else {
                        errorsNb++;
                    }
                    mesuredNb++;
                    totalExecTime += t.execTime;
                    updateTimes(t.processName,t.execTime);
                    printTime(t);
                    long avg = (totalExecTime)/(successNb+errorsNb);
                    printFinished(avg);
                    System.out.println("Finished thread "+t.getName()+", executed in "+t.execTime+" ms, average = "+avg+" ms, currently "+(instancesNb-successNb-errorsNb-notMesuredNb)+" threads running");
                } else {
                    notMesuredNb++;
                    System.out.println("Finished thread "+t.getName()+", executed in "+t.execTime+" ms, currently "+(instancesNb-notMesuredNb)+" threads running");
                }
                if (mode.equals("stress") && currentTime<endTime) {
                    LoadAbstractThread toStart = buildThread(instancesNb++,t.processName);
                    toStart.start();
                }
                int finished = errorsNb+successNb+notMesuredNb;
                int running = instancesNb-finished;
                if (tooManyThreads && running<maxAllowedThreads) {
                    System.out.println("The number of threads is now under "+maxAllowedThreads+", launching a new suite");
                    launchLongSuite();
                    tooManyThreads = false;
                }

                canFinish = finished==instancesNb;
                if (mode.equals("pick")) {
                    canFinish = canFinish && instancesNb == testNb;
                } else if (mode.equals("stress")) {
                    canFinish = canFinish && (currentTime - startMesuringTime)>=loadTime;
                } else if (mode.equals("long")) {
                    canFinish = canFinish && currentLongThreads == finishedLongThreads;
                }
                if (canFinish) {
                    Date d2 = new Date();
                    long diff = d2.getTime()-startMesuringTime;
                    Thread.sleep(3000);
                    printResults(diff);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public synchronized static void launchLongSuite() throws Exception {
        synchronized (mutex) {
            if (!tooManyThreads) {
                finishedLongThreads++;
            }
            int threadNb = instancesNb-errorsNb-successNb;
            if (threadNb<maxAllowedThreads) {
                LongElement le = (LongElement) longList.get(longCpt++);

                launchNextLongThread();

                for (int i=0 ; i<le.getThreadNb();i++) {
                    LoadAbstractThread toStart = buildThread(instancesNb++);
                    toStart.start();
                }
            } else {
                System.out.println("There are currently too many threads in the JVM : "+threadNb+", maxAllowed = "+maxAllowedThreads+", Waiting...");
                tooManyThreads = true;
            }
        }
    }

    public static void launchNextLongThread() {
        long startedSince = new Date().getTime()-startTime;
        int cpt = 0;
        synchronized (mutex) {
            if (longCpt == longList.size()) {
                longCpt = 0;
            }
            cpt = longCpt;

            LongElement le = (LongElement) longList.get(cpt);

            if ((startedSince+le.getWaitTime())<loadTime) {
                System.out.println("Started since : "+startedSince+", launching new long thread");
                currentLongThreads++;
                LongThread lt = new LongThread(le.getWaitTime());
                lt.start();
            }
        }
    }

    public static void readProcessFile() throws Exception {
        processes = new Vector();
        processes.addElement("helloworld");
        processes.addElement("loadInvoke");
        processes.addElement("loan");
    }

    public static void printResults(long totalTime) throws Exception {
        System.out.println("\n"+mesuredNb+" tests run, "+errorsNb+" errors, "+successNb+" success." );
        System.out.println("Total time : "+Util.getTime(totalTime));
        System.out.println("");
        fw.write(Util.EXECTIME+"\n");
        fw.write(Long.toString(totalTime)+"\n");
        fw.close();
        ew.close();

        ReportAnalyser hrw = new ReportAnalyser(file.getAbsolutePath());
        hrw.analyseReport(bestTimes,worthTimes);
    }

    public static LoadAbstractThread buildThread(int i) throws Exception {
        String processName = (String) toLaunch.get(currentProcessNb);
        if ((currentProcessNb+1) == toLaunch.size()) {
            currentProcessNb = 0;
        } else {
            currentProcessNb ++;
        }
        return buildThread(i,processName);
    }

    public static LoadAbstractThread buildThread(int i, String processName) throws Exception {
        LoadAbstractThread thread = null;
        Class cl = Class.forName(Util.basePack+BPELString.toUpper(processName)+"Thread");
        Constructor constr = cl.getConstructor(new Class[]{int.class,int.class});
        thread = (LoadAbstractThread) constr.newInstance(new Object[]{new Integer(i),new Integer(axisTimeOut)});
        System.out.println("Adding thread "+thread.getName());
        return thread;
    }

    public static void printTime(LoadAbstractThread t) throws Exception {
        String sep = " ";
        String st = t.processName+sep+t.nb+sep+t.start+sep+t.execTime+sep+t.success;
        fw.write(st+"\n");
    }

    public static void printFinished(long avg) throws Exception {
        //each timeBetweenPrints miliseconds, print finishedThreads;
        long time = new Date().getTime();
        int finished = errorsNb + successNb;
        if (time> (lastPrint+timeBetweenPrints) || finished == testNb) {
            lastPrint = time;
            System.out.println("");
            System.out.println("Finished Threads ("+finished+") : "+errorsNb+" errors, "+successNb+" success.");
            System.out.println("Started since : "+Util.getTime(time-startTime) + ", Mesuring since : " + Util.getTime(time-startMesuringTime));
            int instPerMin = getInstancesPerMin(time,finished);
            System.out.println("Average number of Instances per minutes : "+instPerMin);
            fw.write(Util.STATS+";"+instPerMin+";"+avg+"\n");
            if ((mode.equals("stress") || mode.equals("long")) && (endTime - time) >0) {
                System.out.println("End in : "+Util.getTime(endTime - time));
            }
            System.out.println("");
        }
    }

    private static int getInstancesPerMin(long currentTime,int finishedInstances) {
        float timeDif = currentTime-startMesuringTime;
        timeDif = timeDif / 60000;
        float result = finishedInstances / timeDif;
        int instPerMin = Math.round(result);
        return instPerMin;
    }

    private static void analyseArgs(String [] args) throws Exception {
        testNb = Integer.parseInt(args[0]);
        mode = args[1];
        time_between_processes = Long.parseLong(args[2]);
        timeBetweenPrints = Long.parseLong(args[3])*1000;
        loadTime = Long.parseLong(args[4]);
        toLaunch = new Vector();
        toLaunchString = args[5];
        readProcessFile();
        if (toLaunchString.equals("all")) {
            toLaunch = processes;
        } else {
            String[] tests = toLaunchString.split(",");
            for (int i=0;i<tests.length;i++) {
                toLaunch.addElement(tests[i]);
            }
        }
        if (toLaunch.size() == 0) {
            throw new Exception("No process in toLaunch processes list");
        }
        output = args[6];
        exitOnError = new Boolean(args[7]).booleanValue();
        longListSt = args[8];

        longList = new Vector();
        String[] longs = longListSt.split(";");
        for (int i=0;i<longs.length;i++) {
            String[] elems = longs[i].split(",");
            longList.addElement(new LongElement(elems[0],elems[1]));
        }

        maxAllowedThreads = Integer.parseInt(args[9]);

        time_to_load = Long.parseLong(args[10]);

        axisTimeOut = Integer.parseInt(args[11]) * 1000;

        System.out.println("Number of tests launched (for pick mode) : "+testNb);
        System.out.println("Mode : "+mode);
        System.out.println("Wait time between two processes : "+time_between_processes);
        System.out.println("Wait time between two prints : "+timeBetweenPrints);
        System.out.println("Time before Mesures : "+time_to_load);
        System.out.println("LoadTime (for stress and long modes) : "+loadTime);
        System.out.println("Processes to launch : "+toLaunchString);
        System.out.println("Output directory : "+output);
        System.out.println("Exit On Error : "+exitOnError);
        System.out.println("LongList : "+longListSt);
        System.out.println("Max Allowed Threads : "+maxAllowedThreads);
        System.out.println("Axis TimeOut : "+axisTimeOut);
        System.out.println("");
    }

    public static void createLogFile() throws Exception {
        String fileName = Util.getDate()+Util.sep+mode;
        if (!mode.equals("long")) {
            fileName += "_"+testNb;
        }
        if (mode.equals("stress") || mode.equals("long")) {
            fileName += "_"+loadTime;
        }
        file = new File(output+File.separator+Util.logFileStart+fileName+Util.logFileExtension);
        file.getAbsoluteFile().getParentFile().mkdirs();
        file.createNewFile();
        fw = new FileWriter(file,true);

        fw.write(Util.PARAMETERS+"\n");
        /*
        fw.write("Number of tests launched (for pick mode)="+testNb+"\n");
        fw.write("Mode="+mode+"\n");
        fw.write("Wait time between two processes="+time_between_processes+"\n");
        fw.write("Wait time between two prints="+timeBetweenPrints+"\n");
        fw.write("LoadTime (for stress and long modes)="+loadTime+"\n");
        fw.write("Processes to launch="+toLaunchString+"\n");
        fw.write("Output directory="+output+"\n");
        fw.write("Exit on  error="+exitOnError+"\n");
        fw.write("Long list="+longListSt+"\n");
        fw.write("MaxAllowedThreads="+maxAllowedThreads+"\n");
        */
        fw.write("testNb="+testNb+"\n");
        fw.write("mode="+mode+"\n");
        fw.write("time_between_processes="+time_between_processes+"\n");
        fw.write("time_to_load="+time_to_load+"\n");
        fw.write("timeBetweenPrints="+timeBetweenPrints+"\n");
        fw.write("loadTime="+loadTime+"\n");
        fw.write("toLaunch="+toLaunchString+"\n");
        fw.write("output="+output+"\n");
        fw.write("exitOnError="+exitOnError+"\n");
        fw.write("longList="+longListSt+"\n");
        fw.write("maxAllowedThreads="+maxAllowedThreads+"\n");
        fw.write("axisTimeOut="+axisTimeOut+"\n");

        fw.write(Util.DATAS+"\n");
    }

    public static void createErrorsFile() throws Exception {
        String fileName = Util.getDate()+Util.sep+"Errors";
        File file = new File(output+File.separator+Util.logFileStart+fileName+Util.logFileExtension);
        file.getAbsoluteFile().getParentFile().mkdirs();
        file.createNewFile();
        ew = new FileWriter(file,true);
        Util.setErrorWriter(ew);
    }

    public static boolean exitOnError() {
        return exitOnError;
    }

}
