package org.objectweb.orchestra.tests.unit.unitCondition;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.objectweb.orchestra.tests.unit.unitCondition.Complex;
import org.objectweb.orchestra.tests.unit.unitCondition.Complex2;
import org.objectweb.orchestra.tests.unit.unitCondition.holders.Complex2Holder;
import org.objectweb.orchestra.tests.unit.unitCondition.ConditionPT;
import org.objectweb.orchestra.tests.unit.unitCondition.ConditionServiceBP;
import org.objectweb.orchestra.tests.unit.unitCondition.ConditionServiceBPLocator;
import javax.xml.rpc.holders.BooleanHolder;

import org.objectweb.orchestra.util.BPELConstants;



public class TestCondition extends TestCase {

    private boolean tmp;

    private ConditionServiceBP fs = null;
    private ConditionPT fpt = null;

    private String cas = null;
    private int integer;
    private String message = null;

    public TestCondition(String testname) {
        super(testname);
    }

    public static TestSuite suite() {
        return new TestSuite(TestCondition.class);
    }

    public void setUp() throws Exception {
        System.out.println(BPELConstants.TEST_MESS_BEGIN+this.getName()+BPELConstants.TEST_MESS_END);
        this.tmp = false;
        this.message = null;
        this.fs = new ConditionServiceBPLocator();
        this.fpt = this.fs.getConditionPT();
    }

    public static void main(String args[]) {
        String testtorun = null;
        // Get args: <test-name>
        if (args.length > 0) {
            testtorun = args[0];
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new TestCondition(testtorun));
        }
    }

    public void testCondition1() throws Exception {
        //condition="0!=      getVariableData('in1','integer')    "
        //condition="   getVariableData('in1','integer')   !=   0           "
        //condition="1     !=getVariableData('in1','integer')  "
        this.cas = "1";
        this.integer = 0;
        this.message = this.fpt.condition1(this.cas,this.integer);
        this.tmp = this.message.equals("condition1");
        assertTrue("condition1 error",this.tmp);
    }

    public void testCondition2() throws Exception {
        //condition="0  &gt;      getVariableProperty('in1','int')"
        //condition=" getVariableProperty('in1','int')     != getVariableProperty('in1','int') "
        //condition="getVariableData('in1','integer')&lt;=getVariableProperty('in1','int')"
        this.cas = "1";
        this.integer = 0;
        this.message = this.fpt.condition2(this.cas,this.integer);
        this.tmp = this.message.equals("condition2");
        assertTrue("condition2 error",this.tmp);
    }

    public void testCondition3() throws Exception {
        //condition="'0'  =      getVariableProperty('in1','string')"
        //condition=" getVariableProperty('in1','string')     != getVariableProperty('in1','string') "
        //condition="    getVariableData('in1','cas')  =   '1'   "
        this.cas = "1";
        this.integer = 0;
        this.message = this.fpt.condition3(this.cas,this.integer);
        this.tmp = this.message.equals("condition3");
        assertTrue("condition3 error",this.tmp);
    }

    public void testCondition4() throws Exception {
        int int0 = 0;
        int int1 = 1;
        int int2 = 2;
        int int3 = 3;
        int int4 = 4;
        String st0 = "st0";
        String st1 = "st1";
        boolean boolean0 = false;
        Complex complex = new Complex();
        complex.setInt0(int0);
        complex.setInt1(int1);
        complex.setInt2(int2);
        complex.setInt3(int3);
        complex.setInt4(int4);
        complex.setString0(st0);
        complex.setString1(st1);
        complex.set_boolean(boolean0);

        Complex complexOut = this.fpt.condition4(complex);

        this.tmp = true;
        this.tmp = this.tmp && complexOut.getString0().equals(" string0   ");
        assertTrue("condition4 error : string0",this.tmp);
        this.tmp = this.tmp && complexOut.getInt0()==int0;
        assertTrue("condition4 error : int0",this.tmp);
        this.tmp = this.tmp && complexOut.getInt1()==int1;
        assertTrue("condition4 error : int1",this.tmp);
        this.tmp = this.tmp && complexOut.getInt2()== -int2+2;
        assertTrue("condition4 error : int2",this.tmp);
        this.tmp = this.tmp && complexOut.getInt3()== int3*2;
        assertTrue("condition4 error : int3",this.tmp);
        this.tmp = this.tmp && complexOut.getInt4()== int4*2;
        assertTrue("condition4 error : int4",this.tmp);
        this.tmp = this.tmp && complexOut.getInt5()== 2-int2;
        assertTrue("condition4 error : int5",this.tmp);
        this.tmp = this.tmp && complexOut.getInt6()== 3+2-1*5*(-1+ -4*2);
        assertTrue("condition4 error : int6",this.tmp);
        this.tmp = this.tmp && complexOut.is_boolean()== true;
        assertTrue("condition4 error : boolean0",this.tmp);
    }

    public void testCondition5() throws Exception {
        int int0 = 0;
        String st0 = "st0";
        boolean boolean0 = false;
        Complex2 complex2 = new Complex2();
        complex2.set_int(int0);
        complex2.setString(st0);
        complex2.set_boolean(boolean0);

        Complex2Holder complex2H = new Complex2Holder();
        BooleanHolder boolH = new BooleanHolder(boolean0);

        this.fpt.condition5(complex2,complex2H,boolH);

        this.tmp = true;

        this.tmp = this.tmp && complex2H.value.getString().equals(" bpws:getVariableData(in3,string)   "+st0);
        assertTrue("condition5 error : string",this.tmp);
        this.tmp = this.tmp && complex2H.value.is_boolean() == boolean0;
        assertTrue("condition5 error : boolean",this.tmp);
        this.tmp = this.tmp && boolH.value == boolean0;
        assertTrue("condition5 error : boolH",this.tmp);
    }

    public void testCondition6() throws Exception {
        String in = "Order doesn't exist";
        String out = this.fpt.condition6(in);
        this.tmp = out.equals("ok");
        assertTrue("condition6 error : out = "+out,this.tmp);
    }

}
