package org.objectweb.orchestra.tests.unit.unitCopy;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.objectweb.orchestra.tests.unit.unitCopy.Copy;
import org.objectweb.orchestra.tests.unit.unitCopy.CopyServiceBP;
import org.objectweb.orchestra.tests.unit.unitCopy.CopyServiceBPLocator;

import org.objectweb.orchestra.tests.unit.unitCopy.Complex;
import org.objectweb.orchestra.tests.unit.unitCopy.SubComplex;
import org.objectweb.orchestra.tests.unit.unitCopy.Expr;
import org.objectweb.orchestra.tests.unit.unitCopy.Expr2;
import org.objectweb.orchestra.tests.unit.unitCopy.Expr3;

import org.objectweb.orchestra.tests.unit.unitCopy.holders.ComplexHolder;
import javax.xml.rpc.holders.IntHolder;
import javax.xml.rpc.holders.StringHolder;
import javax.xml.rpc.holders.BooleanHolder;

import org.objectweb.orchestra.util.BPELConstants;

import org.objectweb.orchestra.tests.unit.unitCopy.holders.SubComplexHolder;


public class TestCopy extends TestCase {

    private static CopyServiceBP se = null;
    private static Copy pt = null;

    private static boolean result = true;

    private static Complex c = null;
    private static int i = -1;
    private static String s = null;
    private static SubComplex sc = null;
    private static boolean b = true;

    private static ComplexHolder ch = null;
    private static IntHolder ih = null;
    private static StringHolder sh = null;
    private static SubComplexHolder sch = null;
    private static BooleanHolder bh = null;

    public TestCopy(String testname) {
        super(testname);
    }

    public static TestSuite suite() {
        return new TestSuite(TestCopy.class);
    }

    public void setUp() throws Exception {
        System.out.println(BPELConstants.TEST_MESS_BEGIN+this.getName()+BPELConstants.TEST_MESS_END);
        result = true;
        se = new CopyServiceBPLocator();
        pt = se.getCopy();
    }

    public static void main(String args[]) {
        String testtorun = null;
        // Get args: <test-name>
        if (args.length > 0) {
            testtorun = args[0];
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new TestCopy(testtorun));
        }
    }

    private static void init() {
        c = new Complex(new Integer(1),new Long(2),"complexString",new SubComplex(new Integer(3),"complex/subComplexString"));
        i = -1;
        s = "stringPart";
        sc = new SubComplex(new Integer(4),"subComplexString");
        b = true;

        ch = new ComplexHolder(c);
        ih = new IntHolder(i);
        sh = new StringHolder(s);
        sch = new SubComplexHolder(sc);
        bh = new BooleanHolder(b);
    }

    public void testCopy1() throws Exception {
        init();
        pt.copy1(ch,ih,sh,sch);
        result = result && ch.value.equals(c) && ih.value == i && sh.value.equals(s) && sch.value.equals(sc);
        assertTrue("copy1 Error",result);
    }

    public void testCopy2() throws Exception {
        init();
        pt.copy2(c,i,s,sc,ch,ih,sh,sch);
        result = result && c.getSubComplex().equals(ch.value.getSubComplex());
        assertTrue("copy2 Error1",result);
        result = result && c.getSubComplex().getString().equals(ch.value.getString());
        assertTrue("copy2 Error2",result);
        result = result && c.getSubComplex().get_int().equals(ch.value.get_int());
        assertTrue("copy2 Error3",result);
        result = result && sc.equals(sch.value);
        assertTrue("copy2 Error4",result);
        result = result && s.equals(sh.value);
        assertTrue("copy2 Error5, s = '"+s+"', sh.value = '"+sh.value+"'",result);
        result = result && i == ih.value;
        assertTrue("copy2 Error6",result);
    }

    public void testCopy3() throws Exception {
        init();
        pt.copy3(i,s,b,ih,sh,bh);
        result = result && ih.value == i;
        assertTrue("copy3 Error1",result);
        result = result && sh.value.equals(s);
        assertTrue("copy3 Error2",result);
        result = result && bh.value == b;
        BooleanHolder b2h = new BooleanHolder();
        b2h.value = true;
        BooleanHolder b3h = new BooleanHolder();
        b3h.value = false;
        assertTrue("copy3 Error3, bh.value = "+bh.value+", b = "+b+", b2h.value = "+b2h.value+", b3h.value = "+b3h.value,result);
    }

    public void testCopy4() throws Exception {
        init();
        Complex cout = pt.copy4(c);
        result = result && cout.equals(c);
        assertTrue("copy4 Error",result);
    }

    public void testCopy5() throws Exception {
        init();
        Complex cout = pt.copy5(c);
        result = result && cout.getSubComplex().equals(c.getSubComplex());
        assertTrue("copy5 Error1",result);
        result = result && cout.get_long().equals(c.get_long());
        assertTrue("copy5 Error2",result);
    }

    public void testCopy6() throws Exception {
        init();
        Expr expr = pt.copy6(c,i,s,sc);
        result = result && expr.getString1().equals("expr1");
        assertTrue("copy6 Error1",result);
        result = result && expr.getString2().equals("expr3expr3bis");
        assertTrue("copy6 Error2",result);
        result = result && expr.getString3().equals("expr4expr1");
        assertTrue("copy6 Error3",result);
        result = result && expr.getString4().equals(c.getString()+"expr5");
        assertTrue("copy6 Error4",result);
        result = result && expr.getString5().equals(c.getSubComplex().getString());
        assertTrue("copy6 Error5",result);
        result = result && expr.getString6().equals(" all expr1hello bye");
        assertTrue("copy6 Error6",result);
        result = result && expr.getString7().equals(" all bpws:getVariableData(hello bye");
        assertTrue("copy6 Error7",result);
        result = result && expr.getString8().equals("3+2*5");
        assertTrue("copy6 Error8",result);

        result = result && expr.getInt1() == 1;
        assertTrue("copy6 Error9",result);
        result = result && expr.getInt2() == 13;
        assertTrue("copy6 Error10",result);
        result = result && expr.get_double() == 2.0;
        assertTrue("copy6 Error11",result);
        result = result && expr.get_long() == 123465789;
        assertTrue("copy6 Error12",result);
        result = result && expr.is_boolean() == true;
        assertTrue("copy6 Error13",result);
    }

    public void testCopy7() throws Exception {
        init();
        Expr2 expr = pt.copy7(i,s,b);
        result = result && expr.getString1().equals(s+s);
        assertTrue("copy7 Error1",result);
        result = result && expr.getString2().equals(s);
        assertTrue("copy7 Error2",result);
    }

    public void testCopy8() throws Exception {
        init();
        Complex cout = pt.copy8("");
        result = result && cout.get_long().longValue() == 10;
        assertTrue("copy8 Error1",result);
        result = result && cout.get_int().intValue() == 11;
        assertTrue("copy8 Error2",result);
    }

    public void testCopy9() throws Exception {
        init();
        Expr3 expr = pt.copy9("");
        result = result && expr.get_long() == 10;
        assertTrue("copy9 Error1",result);
        result = result && expr.getInt1() == 11;
        assertTrue("copy9 Error2",result);
    }

    public void testCopy10() throws Exception {
        init();
        pt.copy10(c,i,s,sc,ch,ih,sh,sch);
        result = result && c.equals(ch.value);
        assertTrue("copy10 Error1", result);
    }

    public void testCopy11() throws Exception {
        init();

        String apos = "'inCopy11_with_apos'";
        String amp = "\"inCopy11_with_amp\"";
        String apos2 = "abcd'efg'hi'j";
        String amp2 = "abcde\"fghi\"jql\"";

        BooleanHolder bh1 = new BooleanHolder();
        BooleanHolder bh2 = new BooleanHolder();
        BooleanHolder bh3 = new BooleanHolder();
        BooleanHolder bh4 = new BooleanHolder();
        BooleanHolder bh5 = new BooleanHolder();
        BooleanHolder bh6 = new BooleanHolder();
        BooleanHolder bh7 = new BooleanHolder();
        BooleanHolder bh8 = new BooleanHolder();
        StringHolder sh9 = new StringHolder();
        StringHolder sh10 = new StringHolder();
        StringHolder sh11 = new StringHolder();
        StringHolder sh12 = new StringHolder();
        pt.copy11(apos,amp,apos2,amp2,bh1,bh2,bh3,bh4,bh5,bh6,bh7,bh8,sh9,sh10,sh11,sh12);
        result = result && bh1.value == true;
        assertTrue("copy11 Error1", result);
        result = result && bh2.value == true;
        assertTrue("copy11 Error2", result);
        result = result && bh3.value == true;
        assertTrue("copy11 Error3", result);
        result = result && bh4.value == true;
        assertTrue("copy11 Error4", result);
        result = result && bh5.value == true;
        assertTrue("copy11 Error5", result);
        result = result && bh6.value == true;
        assertTrue("copy11 Error6", result);
        result = result && bh7.value == true;
        assertTrue("copy11 Error7", result);
        result = result && bh8.value == true;
        assertTrue("copy11 Error8", result);
        result = result && sh9.value.equals(apos2);
        assertTrue("copy11 Error9", result);
        result = result && sh10.value.equals(amp2);
        assertTrue("copy11 Error10", result);
        result = result && sh11.value.equals("'abcd'efg'hi'j");
        assertTrue("copy11 Error11", result);
        result = result && sh12.value.equals("abcde\"\"fghi\"jql\"");
        assertTrue("copy11 Error12", result);




    }
}

