package org.objectweb.orchestra.tests.unit.unitCopyJAXRPC;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.objectweb.orchestra.tests.unit.unitCopyJAXRPC.CopyJAXRPCPT;
import org.objectweb.orchestra.tests.unit.unitCopyJAXRPC.CopyJAXRPCServiceBP;
import org.objectweb.orchestra.tests.unit.unitCopyJAXRPC.CopyJAXRPCServiceBPLocator;


import java.math.BigDecimal;
import java.math.BigInteger;
import javax.xml.namespace.QName;

import org.objectweb.orchestra.util.BPELConstants;

import java.util.Locale;
import java.util.TimeZone;
import java.util.Calendar;
import java.util.GregorianCalendar;

import java.net.URI;

public class TestCopyJAXRPC extends TestCase {

    private boolean tmp;

    private CopyJAXRPCServiceBP fs = null;
    private CopyJAXRPCPT fpt = null;

    public TestCopyJAXRPC(String testname) {
        super(testname);
    }

    public static TestSuite suite() {
        return new TestSuite(TestCopyJAXRPC.class);
    }

    public void setUp() throws Exception {
        System.out.println(BPELConstants.TEST_MESS_BEGIN+this.getName()+BPELConstants.TEST_MESS_END);
        this.tmp = false;
        this.fs = new CopyJAXRPCServiceBPLocator();
        this.fpt = this.fs.getCopyJAXRPCPT();
    }

    public static void main(String args[]) {
        String testtorun = null;
        // Get args: <test-name>
        if (args.length > 0) {
            testtorun = args[0];
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new TestCopyJAXRPC(testtorun));
        }
    }

    public void testCopyJAXRPC1() throws Exception {
        String st = new String("copy1");
        String st2 = this.fpt.copyJAXRPC1(st);
        this.tmp = st.equals(st2);
        assertTrue("copyJAXRPC1 error",this.tmp);
    }

    public void testCopyJAXRPC2() throws Exception {
        BigInteger bi = new BigInteger("-10");
        BigInteger bi2 = this.fpt.copyJAXRPC2(bi);
        this.tmp = bi.equals(bi2);
        assertTrue("copyJAXRPC2 error",this.tmp);
    }

    public void testCopyJAXRPC3() throws Exception {
        int i = 54;
        int i2 = this.fpt.copyJAXRPC3(i);
        this.tmp = i == i2;
        assertTrue("copyJAXRPC3 error",this.tmp);
    }

    public void testCopyJAXRPC4() throws Exception {
        long l = 5;
        long l2 = this.fpt.copyJAXRPC4(l);
        this.tmp = l == l2;
        assertTrue("copyJAXRPC4 error",this.tmp);
    }

    public void testCopyJAXRPC5() throws Exception {
        short s = 4;
        short s2 = this.fpt.copyJAXRPC5(s);
        this.tmp = s == s2;
        assertTrue("copyJAXRPC5 error",this.tmp);
    }

    public void testCopyJAXRPC6() throws Exception {
        BigDecimal bd = new BigDecimal("+15");
        BigDecimal bd2 = this.fpt.copyJAXRPC6(bd);
        this.tmp = bd.equals(bd2);
        assertTrue("copyJAXRPC6 error",this.tmp);
    }

    public void testCopyJAXRPC7() throws Exception {
        float f = 5;
        float f2 = this.fpt.copyJAXRPC7(f);
        this.tmp = f == f2;
        assertTrue("copyJAXRPC7 error",this.tmp);
    }

    public void testCopyJAXRPC8() throws Exception {
        double d = 4;
        double d2 = this.fpt.copyJAXRPC8(d);
        this.tmp = d == d2;
        assertTrue("copyJAXRPC8 error",this.tmp);
    }

    public void testCopyJAXRPC9() throws Exception {
        boolean bool = true;
        boolean bool2 = this.fpt.copyJAXRPC9(bool);
        this.tmp = bool == bool2;
        assertTrue("copyJAXRPC9 error",this.tmp);
    }

    public void testCopyJAXRPC10() throws Exception {
        byte by = 5;
        byte by2 = this.fpt.copyJAXRPC10(by);
        this.tmp = by == by2;
        assertTrue("copyJAXRPC10 error",this.tmp);
    }

    public void testCopyJAXRPC11() throws Exception {
        long ui = 10;
        long ui2 = this.fpt.copyJAXRPC11(ui);
        this.tmp = ui == ui2;
        assertTrue("copyJAXRPC11 error",this.tmp);
    }

    public void testCopyJAXRPC12() throws Exception {
        int us = 10;
        int us2 = this.fpt.copyJAXRPC12(us);
        this.tmp = us == us2;
        assertTrue("copyJAXRPC12 error",this.tmp);
    }

    public void testCopyJAXRPC13() throws Exception {
        short ub = 2;
        short ub2 = this.fpt.copyJAXRPC13(ub);
        this.tmp = ub == ub2;
        assertTrue("copyJAXRPC13 error",this.tmp);
    }

    public void testCopyJAXRPC14() throws Exception {
        QName qn = new QName("tns","toto");
        QName qn2 = this.fpt.copyJAXRPC14(qn);
        this.tmp = qn.equals(qn2);
        assertTrue("copyJAXRPC14 error",this.tmp);
    }

    public void testCopyJAXRPC15() throws Exception {
        Calendar cal = new GregorianCalendar(TimeZone.getTimeZone("GMT"),Locale.FRANCE);
        Calendar cal2 = this.fpt.copyJAXRPC15(cal);
        this.tmp = cal.getTime().equals(cal2.getTime());
        assertTrue("copyJAXRPC15 error",this.tmp);
    }

    public void testCopyJAXRPC16() throws Exception {
        Calendar c = new GregorianCalendar();
        c.setTimeInMillis(System.currentTimeMillis());
        Calendar c2 = this.fpt.copyJAXRPC16(c);
        this.tmp = ((c2.get(Calendar.DAY_OF_MONTH) == c.get(Calendar.DAY_OF_MONTH)) &&
                (c2.get(Calendar.MONTH) == c.get(Calendar.MONTH)) &&
                (c2.get(Calendar.YEAR) == c.get(Calendar.YEAR)));
        assertTrue("copyJAXRPC16 error",this.tmp);
    }

    public void testCopyJAXRPC17() throws Exception {
        //Time time = new Time("12:24:52");
        //Time time2 = fpt.copyJAXRPC17(time);
        //tmp = time.toString().equals(time2.toString());
        Calendar c = new GregorianCalendar(TimeZone.getTimeZone("GMT"),Locale.FRANCE);
        Calendar c2 = this.fpt.copyJAXRPC17(c);
        this.tmp = ((c2.get(Calendar.HOUR_OF_DAY) == c.get(Calendar.HOUR_OF_DAY)) &&
                (c2.get(Calendar.MINUTE) == c.get(Calendar.MINUTE)) &&
                (c2.get(Calendar.SECOND) == c.get(Calendar.SECOND)));
        assertTrue("copyJAXRPC17 error",this.tmp);
    }

    public void testCopyJAXRPC18() throws Exception {
        URI uri = new URI("http://www.google.com");
        URI uri2 = this.fpt.copyJAXRPC18(uri);
        this.tmp = uri.equals(uri2);
        assertTrue("copyJAXRPC18 error",this.tmp);
    }

    public void testCopyJAXRPC19() throws Exception {
        byte[] tabByte = new byte[] {1,3};
        byte[] tabByte2 = this.fpt.copyJAXRPC19(tabByte);
        this.tmp = (tabByte[0] == tabByte2[0]) && (tabByte[1] == tabByte2[1]);
        assertTrue("copyJAXRPC19 error",this.tmp);
    }

    public void testCopyJAXRPC20() throws Exception {
        byte[] tabByte21 = new byte[] {2,4};
        byte[] tabByte22 = this.fpt.copyJAXRPC20(tabByte21);
        this.tmp = (tabByte21[0] == tabByte22[0]) && (tabByte21[1] == tabByte22[1]);
        assertTrue("copyJAXRPC20 error",this.tmp);
    }
}



