package org.objectweb.orchestra.tests.unit.unitCopyJAXRPC4;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.objectweb.orchestra.tests.unit.unitCopyJAXRPC4.CopyJAXRPC4PT;
import org.objectweb.orchestra.tests.unit.unitCopyJAXRPC4.CopyJAXRPC4ServiceBP;
import org.objectweb.orchestra.tests.unit.unitCopyJAXRPC4.CopyJAXRPC4ServiceBPLocator;


import javax.xml.rpc.holders.BigIntegerHolder;
import javax.xml.rpc.holders.StringHolder;

import org.objectweb.orchestra.util.BPELConstants;

import java.math.BigInteger;

public class TestCopyJAXRPC4 extends TestCase {

    private boolean tmp;

    private CopyJAXRPC4ServiceBP fs = null;
    private CopyJAXRPC4PT fpt = null;

    public TestCopyJAXRPC4(String testname) {
        super(testname);
    }

    public static TestSuite suite() {
        return new TestSuite(TestCopyJAXRPC4.class);
    }

    public void setUp() throws Exception {
        System.out.println(BPELConstants.TEST_MESS_BEGIN+this.getName()+BPELConstants.TEST_MESS_END);
        this.tmp = false;
        this.fs = new CopyJAXRPC4ServiceBPLocator();
        this.fpt = this.fs.getCopyJAXRPC4PT();
    }

    public static void main(String args[]) {
        String testtorun = null;
        // Get args: <test-name>
        if (args.length > 0) {
            testtorun = args[0];
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new TestCopyJAXRPC4(testtorun));
        }
    }

    public void testCopyJAXRPC4_21() throws Exception {
        String st = new String("hello");
        StringHolder sth = new StringHolder(st);
        this.fpt.copyJAXRPC4_21(sth);
        this.tmp = st.equals(sth.value);
        assertTrue("copyJAXRPC4_21 error",this.tmp);
    }

    public void testCopyJAXRPC4_22() throws Exception {
        String dur = new String("P3Y2M1DT3H4M2S");
        StringHolder durh = new StringHolder(dur);
        this.fpt.copyJAXRPC4_22(durh);
        this.tmp = dur.equals(durh.value);
        assertTrue("copyJAXRPC4_22 error",this.tmp);
    }

    public void testCopyJAXRPC4_23() throws Exception {
        String ym = new String("2004,06");
        StringHolder ymh = new StringHolder(ym);
        this.fpt.copyJAXRPC4_23(ymh);
        this.tmp = ym.equals(ymh.value);
        assertTrue("copyJAXRPC4_23 error",this.tmp);
    }

    public void testCopyJAXRPC4_24() throws Exception {
        String y = new String("2004");
        StringHolder yh = new StringHolder(y);
        this.fpt.copyJAXRPC4_24(yh);
        this.tmp = y.equals(yh.value);
        assertTrue("copyJAXRPC4_24 error",this.tmp);
    }

    public void testCopyJAXRPC4_25() throws Exception {
        String md = new String("06,18");
        StringHolder mdh = new StringHolder(md);
        this.fpt.copyJAXRPC4_25(mdh);
        this.tmp = md.equals(mdh.value);
        assertTrue("copyJAXRPC4_25 error",this.tmp);
    }

    public void testCopyJAXRPC4_26() throws Exception {
        String d = new String("18");
        StringHolder dh = new StringHolder(d);
        this.fpt.copyJAXRPC4_26(dh);
        this.tmp = d.equals(dh.value);
        assertTrue("copyJAXRPC4_26 error",this.tmp);
    }

    public void testCopyJAXRPC4_27() throws Exception {
        String m = new String("06");
        StringHolder mh = new StringHolder(m);
        this.fpt.copyJAXRPC4_27(mh);
        this.tmp = m.equals(mh.value);
        assertTrue("copyJAXRPC4_27 error",this.tmp);
    }

    public void testCopyJAXRPC4_28() throws Exception {
        String t = new String("correct token");
        StringHolder th = new StringHolder(t);
        this.fpt.copyJAXRPC4_28(th);
        this.tmp = t.equals(th.value);
        assertTrue("copyJAXRPC4_28 error",this.tmp);
    }

    public void testCopyJAXRPC4_29() throws Exception {
        String l = new String("correct language");
        StringHolder lh = new StringHolder(l);
        this.fpt.copyJAXRPC4_29(lh);
        this.tmp = l.equals(lh.value);
        assertTrue("copyJAXRPC4_29 error",this.tmp);
    }

    public void testCopyJAXRPC4_30() throws Exception {
        String n = new String("correct:name");
        StringHolder nh = new StringHolder(n);
        this.fpt.copyJAXRPC4_30(nh);
        this.tmp = n.equals(nh.value);
        assertTrue("copyJAXRPC4_30 error",this.tmp);
    }

    public void testCopyJAXRPC4_31() throws Exception {
        String ncn = new String("correct_ncname");
        StringHolder ncnh = new StringHolder(ncn);
        this.fpt.copyJAXRPC4_31(ncnh);
        this.tmp = ncn.equals(ncnh.value);
        assertTrue("copyJAXRPC4_31 error",this.tmp);
    }

    public void testCopyJAXRPC4_32() throws Exception {
        String id = new String("correct_id");
        StringHolder idh = new StringHolder(id);
        this.fpt.copyJAXRPC4_32(idh);
        this.tmp = id.equals(idh.value);
        assertTrue("copyJAXRPC4_32 error",this.tmp);
    }

    public void testCopyJAXRPC4_33() throws Exception {
        String nmt = new String("ab:4");
        StringHolder nmth = new StringHolder(nmt);
        this.fpt.copyJAXRPC4_33(nmth);
        this.tmp = nmt.equals(nmth.value);
        assertTrue("copyJAXRPC4_33 error",this.tmp);
    }

    public void testCopyJAXRPC4_34() throws Exception {
        String nmts = new String("ab:5");
        StringHolder nmtsh = new StringHolder(nmts);
        this.fpt.copyJAXRPC4_34(nmtsh);
        this.tmp = nmts.equals(nmtsh.value);
        assertTrue("copyJAXRPC4_34 error",this.tmp);
    }

    public void testCopyJAXRPC4_35() throws Exception {
        BigInteger npi = new BigInteger("-4");
        BigIntegerHolder npih = new BigIntegerHolder(npi);
        this.fpt.copyJAXRPC4_35(npih);
        this.tmp = npi.equals(npih.value);
        assertTrue("copyJAXRPC4_35 error",this.tmp);
    }

    public void testCopyJAXRPC4_36() throws Exception {
        BigInteger ni = new BigInteger("-4");
        BigIntegerHolder nih = new BigIntegerHolder(ni);
        this.fpt.copyJAXRPC4_36(nih);
        this.tmp = ni.equals(nih.value);
        assertTrue("copyJAXRPC4_36 error",this.tmp);
    }

    public void testCopyJAXRPC4_37() throws Exception {
        BigInteger nni = new BigInteger("4");
        BigIntegerHolder nnih = new BigIntegerHolder(nni);
        this.fpt.copyJAXRPC4_37(nnih);
        this.tmp = nni.equals(nnih.value);
        assertTrue("copyJAXRPC4_37 error",this.tmp);
    }

    public void testCopyJAXRPC4_38() throws Exception {
        BigInteger ul = new BigInteger("400");
        BigIntegerHolder ulh = new BigIntegerHolder(ul);
        this.fpt.copyJAXRPC4_38(ulh);
        this.tmp = ul.equals(ulh.value);
        assertTrue("copyJAXRPC4_38 error",this.tmp);
    }

    public void testCopyJAXRPC4_39() throws Exception {
        BigInteger pi = new BigInteger("40");
        BigIntegerHolder pih = new BigIntegerHolder(pi);
        this.fpt.copyJAXRPC4_39(pih);
        this.tmp = pi.equals(pih.value);
        assertTrue("copyJAXRPC4_39 error",this.tmp);
    }
}
