/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package jorm.rdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PClassMappingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PMappingStructuresManager;
import org.objectweb.jorm.api.PNameIterator;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.genclass.api.FieldDesc;
import org.objectweb.jorm.genclass.api.GenClassProp;
import org.objectweb.jorm.lib.EmptyPNameIterator;
import org.objectweb.jorm.lib.MultiplePNameIterator;
import org.objectweb.medor.eval.lib.TupleAdapter;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
//import for the rdb mapper
import org.objectweb.jorm.api.PPrefetchTupleCollection;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import org.objectweb.jorm.mapper.rdb.adapter.api.JoinedTable;
import org.objectweb.jorm.mapper.rdb.lib.RdbExtentGenInfos;
import org.objectweb.jorm.mapper.rdb.lib.RdbTupleCollection;
import org.objectweb.jorm.mapper.rdb.lib.PClassMappingDecoder;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;
import org.objectweb.jorm.mapper.rdb.lib.RdbPMappingStructuresManager;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbClassMapping;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbMapping;
import org.objectweb.jorm.mapper.rdb.lib.RdbPPolymorphicClass;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.lib.SelectEvaluator;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.eval.api.ConnectionResources;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.CalculatedParameterOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.query.rdb.lib.BasicRdbStringQueryLeaf;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.datasource.lib.ConnectionFactoryDataStore;
import org.objectweb.medor.expression.lib.BasicBinaryArithmeticOperator;
import org.objectweb.medor.expression.lib.Plus;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.eval.prefetch.api.IntermediaryPrefetchBuffer;
import org.objectweb.medor.eval.prefetch.lib.IntermediaryPrefetchBufferImpl;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.jorm.lib.EncodePName;
import org.objectweb.medor.filter.jorm.lib.CompositePName;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.optim.lib.IndexesGenerator;
import org.objectweb.perseus.persistence.api.ConnectionHolder;
import org.objectweb.perseus.persistence.api.WorkingSet;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Collections;
import java.util.Collection;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.mapper.rdb.genclass.RdbFieldDesc;
import org.objectweb.jorm.mapper.rdb.genclass.RdbGenClassProp;
import org.objectweb.jorm.mapper.rdb.lib.RdbPrefetchablePCM;


// Import clauses required by the object BPIEntityBean
import jorm.BPIEntityBeanAccessor;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

public class BPIEntityBeanMapping
	extends org.objectweb.jonas_ejb.container.jorm.RdbFactory
	implements PClassMapping, PClassMappingCtrl, Loggable
		, PClassMappingDecoder
		, RdbPrefetchablePCM
    {
	protected static final String MAPPER_NAME = "rdb";
	protected static final String PROJECT_NAME = "jonas";
	//Fields
	protected static final String bPUs_NAME = "bPUs";
	protected static final String creationDate_NAME = "creationDate";
	protected static final String id_NAME = "id";
	protected static final String messages_NAME = "messages";
	protected static final String partnerLinks_NAME = "partnerLinks";
	protected static final String process_NAME = "process";
	protected static final String processName_NAME = "processName";
	protected static final String scopes_NAME = "scopes";
	protected static final String state_NAME = "state";
	protected static final String targetNamespace_NAME = "targetNamespace";
	protected static final String variables_NAME = "variables";
	//Hidden fields
	protected static final String process_id_NAME = "process_id";

	private final static String[][] SUPER_TYPES =  {};

	/** Fully quailied name of the persitent class */
	protected final static String JORM_CLASS_NAME =
		"jorm.BPIEntityBean";
	/** The actual binder which does the binding action */
	private PBinder binder;
	private PNameCoder classPNC;

	/** The meta object of the current jorm class */
	private Class moc;
	/** The mapper linked to this PClassMapping */
	private PMapper mapper;
	public PNameCoder bPUsPnc;
	PClassMapping bPUsGCM;
	public PNameCoder messagesPnc;
	PClassMapping messagesGCM;
	public PNameCoder partnerLinksPnc;
	PClassMapping partnerLinksGCM;
	public PNameCoder processPnc;
	public PNameCoder scopesPnc;
	PClassMapping scopesGCM;
	public PNameCoder variablesPnc;
	PClassMapping variablesGCM;
	/** the PClassMapping of the sub type */
	private PClassMapping[] subPCMs = null;

	//variables for the resolve method
	private int beginIndex = 0;
	private QueryTree queryTree = null;
	private FieldOperand[] fieldOperands = null;
	private String[] compositeFieldNames = null;
 	/**
	 * The logger instance used by this XMapping instance
	 * and the XBinding instances
	 */
	public Logger logger;
	public LoggerFactory loggerFactory;

	public BPIEntityBeanMapping() {
 	}
	public PNameCoder getBPUsPNameCoder() {
	    return bPUsPnc;
	}
	public PNameCoder getMessagesPNameCoder() {
	    return messagesPnc;
	}
	public PNameCoder getPartnerLinksPNameCoder() {
	    return partnerLinksPnc;
	}
	public PNameCoder getProcessPNameCoder() {
	    return processPnc;
	}
	public PNameCoder getScopesPNameCoder() {
	    return scopesPnc;
	}
	public PNameCoder getVariablesPNameCoder() {
	    return variablesPnc;
	}

	// IMPLEMENTATION OF METHODS FROM THE PClassMappingCtrl INTERFACE

    public void setPNameCoder(PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setPNameCoder(String fn, PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		if (bPUs_NAME.equals(fn)) {
			bPUsPnc = (PNamingContext) pnc;
			return;
		}
		if (messages_NAME.equals(fn)) {
			messagesPnc = (PNamingContext) pnc;
			return;
		}
		if (partnerLinks_NAME.equals(fn)) {
			partnerLinksPnc = (PNamingContext) pnc;
			return;
		}
		if (process_NAME.equals(fn)) {
			processPnc = (PNamingContext) pnc;
			return;
		}
		if (scopes_NAME.equals(fn)) {
			scopesPnc = (PNamingContext) pnc;
			return;
		}
		if (variables_NAME.equals(fn)) {
			variablesPnc = (PNamingContext) pnc;
			return;
		}
		throw new UnsupportedOperationException("This class does'nt contain field reference: " + fn);
    }

    public void setGenClassMapping(PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setGenClassMapping(String fn, PClassMapping pcm)
            throws UnsupportedOperationException {
		if (bPUs_NAME.equals(fn)) {
			bPUsGCM = pcm;
			return;
		}
		if (messages_NAME.equals(fn)) {
			messagesGCM = pcm;
			return;
		}
		if (partnerLinks_NAME.equals(fn)) {
			partnerLinksGCM = pcm;
			return;
		}
		if (scopes_NAME.equals(fn)) {
			scopesGCM = pcm;
			return;
		}
		if (variables_NAME.equals(fn)) {
			variablesGCM = pcm;
			return;
		}
		throw new UnsupportedOperationException("This class does not contain a " + fn + " field referencing a generic class");
    }

    public void setProjectName(String projectname) {
    }


	// IMPLEMENTATION OF METHODS FROM THE PClassMapping INTERFACE

	public void init(PMappingCallback m, MetaObject metaclass) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		mapper = m;
		moc = (Class) metaclass;
		//the getPType method inits the ptype if not already done
		PType ptype = mapper.getPTypeSpace().createPType(
				"jorm.BPIEntityBean",
				SUPER_TYPES);
		setPType(ptype);
        // PType Assignement to the PNC managing the reference process
        String[][] processICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            "jorm.BpelProcessBean",
            processICDef);
        processPnc.setPType(ptype);
        
         try {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}

         // Initialization of the gen class /jorm.BPIEntityBean/messages
        FieldDesc[] fds = null;
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BPIEntityBean_id", "", PTypeSpace.OBJINT, "BPIEntityBean_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) messagesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) messagesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.MessageBean_id", "", PTypeSpace.OBJINT, "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) messagesGCM).setElemDesc(fds);
			((RdbGenClassProp) messagesGCM).setTableName("bpelMessage");
			((RdbGenClassProp) messagesGCM).setColocated(true);
			((RdbGenClassProp) messagesGCM).setColocatedMaster(false);
			((RdbGenClassProp) messagesGCM).setReadOnly(true);
			((RdbGenClassProp) messagesGCM).setTypeConverter(adapter);
        String[] messagesGCDef = {"java.util.Collection"};
        String[][] messagesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            messagesGCDef,
            "jorm.MessageBean",
            messagesICDef);
        ((GenClassProp) messagesGCM).setPType(ptype);
        messagesPnc.setPType(ptype);
        messagesGCM.init(m, null);

         // Initialization of the gen class /jorm.BPIEntityBean/scopes
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BPIEntityBean_id", "", PTypeSpace.OBJINT, "BPIEntityBean_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) scopesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) scopesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.ScopeBean_id", "", PTypeSpace.OBJINT, "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) scopesGCM).setElemDesc(fds);
			((RdbGenClassProp) scopesGCM).setTableName("bpelScope");
			((RdbGenClassProp) scopesGCM).setColocated(true);
			((RdbGenClassProp) scopesGCM).setColocatedMaster(false);
			((RdbGenClassProp) scopesGCM).setReadOnly(true);
			((RdbGenClassProp) scopesGCM).setTypeConverter(adapter);
        String[] scopesGCDef = {"java.util.Collection"};
        String[][] scopesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            scopesGCDef,
            "jorm.ScopeBean",
            scopesICDef);
        ((GenClassProp) scopesGCM).setPType(ptype);
        scopesPnc.setPType(ptype);
        scopesGCM.init(m, null);

         // Initialization of the gen class /jorm.BPIEntityBean/partnerLinks
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BPIEntityBean_id", "", PTypeSpace.OBJINT, "BPIEntityBean_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) partnerLinksGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) partnerLinksGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.PartnerLinkBean_id", "", PTypeSpace.OBJINT, "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) partnerLinksGCM).setElemDesc(fds);
			((RdbGenClassProp) partnerLinksGCM).setTableName("bpelPartnerLink");
			((RdbGenClassProp) partnerLinksGCM).setColocated(true);
			((RdbGenClassProp) partnerLinksGCM).setColocatedMaster(false);
			((RdbGenClassProp) partnerLinksGCM).setReadOnly(true);
			((RdbGenClassProp) partnerLinksGCM).setTypeConverter(adapter);
        String[] partnerLinksGCDef = {"java.util.Collection"};
        String[][] partnerLinksICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            partnerLinksGCDef,
            "jorm.PartnerLinkBean",
            partnerLinksICDef);
        ((GenClassProp) partnerLinksGCM).setPType(ptype);
        partnerLinksPnc.setPType(ptype);
        partnerLinksGCM.init(m, null);

         // Initialization of the gen class /jorm.BPIEntityBean/variables
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BPIEntityBean_id", "", PTypeSpace.OBJINT, "BPIEntityBean_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) variablesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) variablesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.VariableBean_id", "", PTypeSpace.OBJINT, "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) variablesGCM).setElemDesc(fds);
			((RdbGenClassProp) variablesGCM).setTableName("bpelVariable");
			((RdbGenClassProp) variablesGCM).setColocated(true);
			((RdbGenClassProp) variablesGCM).setColocatedMaster(false);
			((RdbGenClassProp) variablesGCM).setReadOnly(true);
			((RdbGenClassProp) variablesGCM).setTypeConverter(adapter);
        String[] variablesGCDef = {"java.util.Collection"};
        String[][] variablesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            variablesGCDef,
            "jorm.VariableBean",
            variablesICDef);
        ((GenClassProp) variablesGCM).setPType(ptype);
        variablesPnc.setPType(ptype);
        variablesGCM.init(m, null);

         // Initialization of the gen class /jorm.BPIEntityBean/bPUs
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BPIEntityBean_id", "", PTypeSpace.OBJINT, "BPIEntityBean_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bPUsGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bPUsGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BPUEntityBean_id", "", PTypeSpace.OBJINT, "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bPUsGCM).setElemDesc(fds);
			((RdbGenClassProp) bPUsGCM).setTableName("bpelBPU");
			((RdbGenClassProp) bPUsGCM).setColocated(true);
			((RdbGenClassProp) bPUsGCM).setColocatedMaster(false);
			((RdbGenClassProp) bPUsGCM).setReadOnly(true);
			((RdbGenClassProp) bPUsGCM).setTypeConverter(adapter);
        String[] bPUsGCDef = {"java.util.Collection"};
        String[][] bPUsICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bPUsGCDef,
            "jorm.BPUEntityBean",
            bPUsICDef);
        ((GenClassProp) bPUsGCM).setPType(ptype);
        bPUsPnc.setPType(ptype);
        bPUsGCM.init(m, null);
 		//register the structures and dependencies
		init(mapper.getPMappingStructuresManager());
        } catch (Exception e) {
			if(e instanceof RdbAdapterException)
				throw new PException(e, "Column type computing error: ");
			else
				e.printStackTrace();
		} 
			computeReadQuery();
		try {
			computeExtentQuery();
		} catch (ExpressionException e) {
			throw new PException(e);
		}
		computePrefetchedExtentQuery();
		}

	public void init(PMappingStructuresManager pmsm) throws PException {
		pmsm.declareClass(JORM_CLASS_NAME);
		try {
        RdbPMappingStructuresManager msm = (RdbPMappingStructuresManager) pmsm;
        if (mapper == null) {
	       	mapper = msm.getPMapper();
	    }
	    if (adapter == null) {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}
		}
	    // Initialization of the structures of the class jorm.BPIEntityBean
			        	msm.addTableColumn(JORM_CLASS_NAME, "bpelBPI", "targetNamespace", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelBPI", "processName", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelBPI", "creationDate", adapter.getSqlType(PTypeSpace.DATE.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelBPI", "state", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelBPI", "BpelProcessBean_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelBPI", "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, -1, -1), true, true, true);
	    		        // Initialization of the structures of the gen class /jorm.BPIEntityBean/messages
		        	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "BPIEntityBean_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BPIEntityBean/scopes
		        	msm.addTableColumn(JORM_CLASS_NAME, "bpelScope", "BPIEntityBean_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "bpelScope", "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BPIEntityBean/partnerLinks
		        	msm.addTableColumn(JORM_CLASS_NAME, "bpelPartnerLink", "BPIEntityBean_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "bpelPartnerLink", "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BPIEntityBean/variables
		        	msm.addTableColumn(JORM_CLASS_NAME, "bpelVariable", "BPIEntityBean_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "bpelVariable", "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BPIEntityBean/bPUs
		        	msm.addTableColumn(JORM_CLASS_NAME, "bpelBPU", "BPIEntityBean_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "bpelBPU", "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
						//dependencies
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BPUEntityBean");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BpelProcessBean");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.VariableBean");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.PartnerLinkBean");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.ScopeBean");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.MessageBean");
		} catch(Exception e) {
			throw new PException("Error while registering mapping structures: " + e.getMessage());
		}
		classDefined(pmsm);
	}
	
	protected void classDefined() throws PException {
		//register class in the cluster
        classDefined(mapper.getPMappingStructuresManager());
	}

	public void classDefined(PMappingStructuresManager pmsm) throws PException {
		if (pmsm == null) {
			throw new PException("PMappingStructureManager is null: " + JORM_CLASS_NAME + " cannot be registered in a cluster.");
		}
		//register class in the cluster
        pmsm.classDefined(JORM_CLASS_NAME);
	}
	
	public PMapper getPMapper() {
		return mapper;
	}

	public PBinder getPBinder() {
		return binder;
	}

	public PNameCoder getClassPNameCoder() {
		return classPNC;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains references*/
	public PNameCoder getPNameCoder()
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PNameCoder getPNameCoder(String fn)
		throws UnsupportedOperationException {
			if (bPUs_NAME.equals(fn))
				return bPUsPnc;
			if (messages_NAME.equals(fn))
				return messagesPnc;
			if (partnerLinks_NAME.equals(fn))
				return partnerLinksPnc;
			if (process_NAME.equals(fn))
				return processPnc;
			if (scopes_NAME.equals(fn))
				return scopesPnc;
			if (variables_NAME.equals(fn))
				return variablesPnc;
		throw new UnsupportedOperationException("This class does'nt contain field reference " + fn);
	}

	public String getClassName() {
		return JORM_CLASS_NAME;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains another gen class*/
	public PClassMapping getGenClassMapping() throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PClassMapping getGenClassMapping(String fn)
		throws UnsupportedOperationException {
			if (bPUs_NAME.equals(fn))
				return bPUsGCM;
			if (messages_NAME.equals(fn))
				return messagesGCM;
			if (partnerLinks_NAME.equals(fn))
				return partnerLinksGCM;
			if (scopes_NAME.equals(fn))
				return scopesGCM;
			if (variables_NAME.equals(fn))
				return variablesGCM;
			throw new UnsupportedOperationException("This class does not contain a " + fn + " field which would reference a generic class");
	}

	public boolean isConform(String mapperName) {
		return mapperName.startsWith(MAPPER_NAME);
	}

	public String getProjectName() {
		return PROJECT_NAME;
	}

	public MetaObject getMetaInfo() {
		return moc;
	}

	public String[][] getSuperClassNames() {
		return SUPER_TYPES;
	}

	public void setPBinder(PBinder b) {
		binder = b;
	}

	public void setClassPNameCoder(PNameCoder pnc) throws PException {
		classPNC = pnc;
	}

	public PBinding createPBinding() throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		PBinding binding = new jorm.BPIEntityBeanBinding();
		binding.init(this);
		return binding;
	}
	/** @deprecated */
    public Iterator getPNameIterator(Object conn) throws PException {
		return getPNameIterator(conn, false, false, null);
    }

	public PNameIterator getPNameIterator(Object conn,
	                                      boolean withSubType,
	                                      boolean prefetching,
	                                      Object txctx) throws PException {
		PNameIterator it = null;
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String mapperName = mapper.getMapperName();
			if(mapperName.indexOf(".") != -1)
				mapperName = mapperName.substring(0, mapperName.indexOf("."));	
			RdbMapping rdbm = (RdbMapping) mapper.getMetaInfoManager().getClass(this.getClassName()).getClassProject(this.getProjectName()).getMapping(mapperName);
			RdbClassMapping rdbcm = (RdbClassMapping) rdbm.getClassMapping();
			String queryToExecute = "";
			int beginIndex = 0;
				queryToExecute = (prefetching ? prefetchedExtentQuery : extentQuery);
				//System.out.println("query to execute:" + queryToExecute);
				//compute the begining index for the PName: number of fields to select + 1
				beginIndex = 6 + 1;
			PreparedStatement ps = connection.prepareStatement(queryToExecute);
			it = new BPIEntityBeanResultIterator(txctx, ps, ps.executeQuery(), binder, prefetching, withSubType, logger, beginIndex, this);
        } catch (SQLException e) {
            throw new PExceptionIO(e, "SQL problem while computing the PName Iterator: class <BPIEntityBean>.");
        } catch (MedorException e) {
            throw new PExceptionIO(e, "Medor problem while initializing the PName Iterator: class <BPIEntityBean>.");
        }
		return it;
	}

	public PClassMapping[] getSubPCMs() throws PException {
		PType ptype = getPType();
		if (ptype == null) {
			throw new PException("No PType defined for the PClassMapping of the class" + JORM_CLASS_NAME);
		}
		PType[] subTypes = ptype.getSubTypes();
		if (subTypes == null || subTypes.length == 0) {
			//No direct subType are mapped then return null
			return null;
		}
		if (subPCMs == null || subPCMs.length != subTypes.length) {
			//compute the PCM array
			subPCMs = new PClassMapping[subTypes.length];
			for (int i=0; i<subTypes.length; i++) {
				subPCMs[i] = mapper.lookup(subTypes[i].getJormName());
			}
		}
		return subPCMs;
	}
	
	// ADD YO //
	public synchronized PName resolve(Object conn, PName pname) throws PException {
				//mapper rdb but not polymorphic
		return pname;
	 	}
	
	public boolean match(Object obj, boolean intermediaryTuple) throws PException {
		return false;
	}
	
	public PName getDecodedPName(TupleCollection tc, PName pname, boolean intermediaryTuple) throws PException {
		PBinder binder = null;
	//test if the tuple is an instance of this class
	if (this.match(tc, intermediaryTuple)) {
		binder  = this.getPBinder();
	} else {
		//test for all the children
		boolean match = false;
		for (int i = 0; i<subPCMs.length && !match; i++) {
				match = subPCMs[i].match(tc, intermediaryTuple);
				binder = subPCMs[i].getPBinder();
		}
		//if no pcm found, error
		if (!match)
			throw new PException("Problem with Medor while resolving pname on the PCM " + JORM_CLASS_NAME + ". No class found for the pname " + pname.toString() + ".");
	}
			return binder.decodeOint(pname.encodeOint());
		}
	// END ADD YO //
	
    public void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException {
        PClassMapping gcm = null;
        bPUsPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bPUs_NAME, new String[]{"java.util.Collection"});
        bPUsGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bPUs_NAME, new String[]{"java.util.Collection"}, "jorm.BPUEntityBean");
        gcm = bPUsGCM;
        messagesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, messages_NAME, new String[]{"java.util.Collection"});
        messagesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, messages_NAME, new String[]{"java.util.Collection"}, "jorm.MessageBean");
        gcm = messagesGCM;
        partnerLinksPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, partnerLinks_NAME, new String[]{"java.util.Collection"});
        partnerLinksGCM = rc.getGenClassMapping(JORM_CLASS_NAME, partnerLinks_NAME, new String[]{"java.util.Collection"}, "jorm.PartnerLinkBean");
        gcm = partnerLinksGCM;
		processPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, process_NAME, "jorm.BpelProcessBean");
        scopesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, scopes_NAME, new String[]{"java.util.Collection"});
        scopesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, scopes_NAME, new String[]{"java.util.Collection"}, "jorm.ScopeBean");
        gcm = scopesGCM;
        variablesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, variables_NAME, new String[]{"java.util.Collection"});
        variablesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, variables_NAME, new String[]{"java.util.Collection"}, "jorm.VariableBean");
        gcm = variablesGCM;
    }

	public PType getPType() {
		return  binder.getPType();
	}

	public void setPType(PType pt) {
		binder.setPType(pt);
	}


	public void write(PBinding pb, Object conn, PAccessor pa)
		throws PException {
		BPIEntityBeanAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof BPIEntityBeanAccessor)
			paccessor = (BPIEntityBeanAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized WRITE operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String sqlquery;
		if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Remove");
//Table bpelBPI : readOnly=false mustGenerateUpdate=true colocatedTable=false joins=$tab.joins
                sqlquery = deletebpelBPI10Table(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BPIEntityBean]: problematic SQL query -> {" + sqlquery + "}");
                }
		} else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Insert");
                sqlquery = insertbpelBPI10Table(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BPIEntityBean]: problematic SQL query -> {" + sqlquery + "}");
                }
		} else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Update");
                sqlquery = updatebpelBPI10Table(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BPIEntityBean]: problematic SQL query -> {" + sqlquery + "}");
                }
		}
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se, "SQL problem while writing the DSI.");
		} catch (java.io.IOException ioe) {
			throw new PExceptionIO(ioe, "Stream IO problem while writing the DSI.");
		} catch (RdbAdapterException rae) {
			throw new PExceptionIO(rae, "SQL problem while writing the DSI.");
		}
		// Changing the current state
		((PBindingCtrl) pb).setStatus(nextState);
	}
	public void read(PBinding pb, Object conn, PAccessor pa) throws PException {
		read(pb, conn, pa, false);
	}
	
	public void read(PBinding pb, Object conn, PAccessor pa, boolean forUpdate) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Read");
		byte status = pb.getStatus();
		BPIEntityBeanAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof BPIEntityBeanAccessor)
			paccessor = (BPIEntityBeanAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet resultSet = null;
        PException resex = null;
		try {
            String jormrq = readQuery;
            if (forUpdate) {
            	jormrq = jormrq + " FOR UPDATE";
            }
            pstmt = connection.prepareStatement(jormrq);
            adapter.setOint(pstmt, 1, pname.encodeOint());
            resultSet = pstmt.executeQuery();
            if(resultSet.next() == false) {
                throw new PExceptionNoDSI(
                    "No entry matching with the following identifier: " 
                    + pname + ", query=" + jormrq);
            }
            PName refname;
            Object fieldval;
            long notnullval;
			if (paccessor != null) {
				//read public primitive fields
                paccessor.paSetCreationDate(adapter.getDate(resultSet, 2, null, null));
                paccessor.paSetId(adapter.getOint(resultSet, 3, null));
                paccessor.paSetProcessName(adapter.getString(resultSet, 4, null));
                paccessor.paSetState(adapter.getString(resultSet, 5, null));
                paccessor.paSetTargetNamespace(adapter.getString(resultSet, 6, null));
				//decode PName for references
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@656758 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@163de4e / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@1ee145b / fieldName=variables / join=null
				//fieldName=id
                fieldval = adapter.getOint(resultSet, 3, null);
                if (resultSet.wasNull()) {
                    refname = variablesPnc.decodeOint(null);
                } else {
                    refname = variablesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessor.paSetVariables(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@197f158 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@163de4e / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@1cda81e / fieldName=partnerLinks / join=null
				//fieldName=id
                fieldval = adapter.getOint(resultSet, 3, null);
                if (resultSet.wasNull()) {
                    refname = partnerLinksPnc.decodeOint(null);
                } else {
                    refname = partnerLinksPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessor.paSetPartnerLinks(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@65d588 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@163de4e / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@f5331a / fieldName=scopes / join=null
				//fieldName=id
                fieldval = adapter.getOint(resultSet, 3, null);
                if (resultSet.wasNull()) {
                    refname = scopesPnc.decodeOint(null);
                } else {
                    refname = scopesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessor.paSetScopes(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1a5f7d3 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1316264 / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@17a687b / fieldName=process / join=null
				//fieldName=process_id
                fieldval = adapter.getOint(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = processPnc.decodeOint(null);
                } else {
                    refname = processPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessor.paSetProcess(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@132c515 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@163de4e / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@b1404f / fieldName=bPUs / join=null
				//fieldName=id
                fieldval = adapter.getOint(resultSet, 3, null);
                if (resultSet.wasNull()) {
                    refname = bPUsPnc.decodeOint(null);
                } else {
                    refname = bPUsPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessor.paSetBPUs(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@291fc2 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@163de4e / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@d86c58 / fieldName=messages / join=null
				//fieldName=id
                fieldval = adapter.getOint(resultSet, 3, null);
                if (resultSet.wasNull()) {
                    refname = messagesPnc.decodeOint(null);
                } else {
                    refname = messagesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessor.paSetMessages(refname, conn);
			} else if (paccessorGen != null) {
				//read public primitive fields
                paccessorGen.paSetDateField(creationDate_NAME, adapter.getDate(resultSet, 2, null, null));
                paccessorGen.paSetOintField(id_NAME, adapter.getOint(resultSet, 3, null));
                paccessorGen.paSetStringField(processName_NAME, adapter.getString(resultSet, 4, null));
                paccessorGen.paSetStringField(state_NAME, adapter.getString(resultSet, 5, null));
                paccessorGen.paSetStringField(targetNamespace_NAME, adapter.getString(resultSet, 6, null));
				//decode PName for references
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@291fc2 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@163de4e / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@d86c58 / fieldName=messages / join=null
				//fieldName=id
                fieldval = adapter.getOint(resultSet, 3, null);
                if (resultSet.wasNull()) {
                    refname = messagesPnc.decodeOint(null);
                } else {
                    refname = messagesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessorGen.paSetRefField(messages_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@291fc2 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@163de4e / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@d86c58 / fieldName=messages / join=null
				//fieldName=id
                fieldval = adapter.getOint(resultSet, 3, null);
                if (resultSet.wasNull()) {
                    refname = messagesPnc.decodeOint(null);
                } else {
                    refname = messagesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessorGen.paSetRefField(messages_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@291fc2 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@163de4e / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@d86c58 / fieldName=messages / join=null
				//fieldName=id
                fieldval = adapter.getOint(resultSet, 3, null);
                if (resultSet.wasNull()) {
                    refname = messagesPnc.decodeOint(null);
                } else {
                    refname = messagesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessorGen.paSetRefField(messages_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@291fc2 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@163de4e / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@d86c58 / fieldName=messages / join=null
				//fieldName=id
                fieldval = adapter.getOint(resultSet, 3, null);
                if (resultSet.wasNull()) {
                    refname = messagesPnc.decodeOint(null);
                } else {
                    refname = messagesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessorGen.paSetRefField(messages_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@291fc2 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@163de4e / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@d86c58 / fieldName=messages / join=null
				//fieldName=id
                fieldval = adapter.getOint(resultSet, 3, null);
                if (resultSet.wasNull()) {
                    refname = messagesPnc.decodeOint(null);
                } else {
                    refname = messagesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessorGen.paSetRefField(messages_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@291fc2 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@163de4e / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@d86c58 / fieldName=messages / join=null
				//fieldName=id
                fieldval = adapter.getOint(resultSet, 3, null);
                if (resultSet.wasNull()) {
                    refname = messagesPnc.decodeOint(null);
                } else {
                    refname = messagesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessorGen.paSetRefField(messages_NAME, refname, conn);
			}
		} catch (PException pe) {
			resex = pe;
		} catch (Exception pe) {
			resex = new PExceptionIO(pe, "Problem while reading the DSI.");
		} finally {
		    try {
		        if (resultSet != null) {
			        resultSet.close();
			        resultSet = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
			    	resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		((PBindingCtrl) pb).setStatus(nextState);
	}
	public void read(PBinding pb, Object conn, PAccessor pa, Object txctx) throws PException {
		read(pb, conn, pa, txctx, false);
	}

	public void read(PBinding pb, Object conn, PAccessor pa, Object txctx, boolean forUpdate) throws PException {
	    PrefetchCache pc = getPMapper().getPrefetchCache();
	    if (pc == null || txctx == null) {
	        read(pb, conn, pa, forUpdate);
	        return;
	    }
		PName pname = pb.getPName();
	    try {
	        Tuple tuple = pc.getPrefetchTuple(pname, this, txctx);
	        if (tuple == null) {
	            read(pb, conn, pa, forUpdate);
	            return;
	        }
		    BPIEntityBeanAccessor paccessor = null;
		    PAccessorGen paccessorGen = null;
		    if (pa == null)
			    throw new PExceptionProtocol("Accessor must not be null");
		    else if (pa instanceof BPIEntityBeanAccessor)
			    paccessor = (BPIEntityBeanAccessor) pa;
		    else if (pa instanceof PAccessorGen)
			    paccessorGen = (PAccessorGen) pa;
		    else
			    throw new PExceptionProtocol("Wrong type of accessor: " + pa);
			byte status = pb.getStatus();
		    byte nextState = PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		    if (nextState==PBinding.LIFECYCLE_ERROR)
			    throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
            Object fval;
            long notnullval;
            if (paccessor != null) {
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 3, PTypeSpace.OBJINT));
                if (tuple.isDefined(3)) {
                    paccessor.paSetBPUs(bPUsPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessor.paSetBPUs(bPUsPnc.getNull(), conn);
                }
                paccessor.paSetCreationDate(tuple.getDate(2));
                paccessor.paSetId(((java.lang.Integer) TupleAdapter.getObject(tuple, 3, PTypeSpace.OBJINT)));
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 3, PTypeSpace.OBJINT));
                if (tuple.isDefined(3)) {
                    paccessor.paSetMessages(messagesPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessor.paSetMessages(messagesPnc.getNull(), conn);
                }
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 3, PTypeSpace.OBJINT));
                if (tuple.isDefined(3)) {
                    paccessor.paSetPartnerLinks(partnerLinksPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessor.paSetPartnerLinks(partnerLinksPnc.getNull(), conn);
                }
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 1, PTypeSpace.OBJINT));
                if (tuple.isDefined(1)) {
                    paccessor.paSetProcess(processPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessor.paSetProcess(processPnc.getNull(), conn);
                }
                paccessor.paSetProcessName(tuple.getString(4));
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 3, PTypeSpace.OBJINT));
                if (tuple.isDefined(3)) {
                    paccessor.paSetScopes(scopesPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessor.paSetScopes(scopesPnc.getNull(), conn);
                }
                paccessor.paSetState(tuple.getString(5));
                paccessor.paSetTargetNamespace(tuple.getString(6));
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 3, PTypeSpace.OBJINT));
                if (tuple.isDefined(3)) {
                    paccessor.paSetVariables(variablesPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessor.paSetVariables(variablesPnc.getNull(), conn);
                }
            } else if (paccessorGen != null) {
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 3, PTypeSpace.OBJINT));
                if (tuple.isDefined(3)) {
                    paccessorGen.paSetRefField(bPUs_NAME, bPUsPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessorGen.paSetRefField(bPUs_NAME, bPUsPnc.getNull(), conn);
                }
                paccessorGen.paSetDateField(creationDate_NAME, tuple.getDate(2));
                paccessorGen.paSetOintField(id_NAME, ((java.lang.Integer) TupleAdapter.getObject(tuple, 3, PTypeSpace.OBJINT)));
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 3, PTypeSpace.OBJINT));
                if (tuple.isDefined(3)) {
                    paccessorGen.paSetRefField(messages_NAME, messagesPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessorGen.paSetRefField(messages_NAME, messagesPnc.getNull(), conn);
                }
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 3, PTypeSpace.OBJINT));
                if (tuple.isDefined(3)) {
                    paccessorGen.paSetRefField(partnerLinks_NAME, partnerLinksPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessorGen.paSetRefField(partnerLinks_NAME, partnerLinksPnc.getNull(), conn);
                }
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 1, PTypeSpace.OBJINT));
                if (tuple.isDefined(1)) {
                    paccessorGen.paSetRefField(process_NAME, processPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessorGen.paSetRefField(process_NAME, processPnc.getNull(), conn);
                }
                paccessorGen.paSetStringField(processName_NAME, tuple.getString(4));
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 3, PTypeSpace.OBJINT));
                if (tuple.isDefined(3)) {
                    paccessorGen.paSetRefField(scopes_NAME, scopesPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessorGen.paSetRefField(scopes_NAME, scopesPnc.getNull(), conn);
                }
                paccessorGen.paSetStringField(state_NAME, tuple.getString(5));
                paccessorGen.paSetStringField(targetNamespace_NAME, tuple.getString(6));
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 3, PTypeSpace.OBJINT));
                if (tuple.isDefined(3)) {
                    paccessorGen.paSetRefField(variables_NAME, variablesPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessorGen.paSetRefField(variables_NAME, variablesPnc.getNull(), conn);
                }
            }
			((PBindingCtrl) pb).setStatus(nextState);
		} catch (MedorException me) {
		    throw new PExceptionProtocol(me, "Problem while trying to prefetch data from Medor");
		}
    }

	public boolean exist(PBinding pb, Object conn) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Exist");
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized EXIST operation, status=" + status);
		((PBindingCtrl) pb).setStatus(nextState);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        PException resex = null;
        boolean exist = false;
		try {
            pstmt = connection.prepareStatement("SELECT bpelBPI.id FROM bpelBPI WHERE bpelBPI.id = ?");
            adapter.setOint(pstmt, 1, pname.encodeOint());
			rs = pstmt.executeQuery();
            exist = rs.next();
		} catch (java.sql.SQLException se) {
			resex = new PExceptionIO(se, "SQL problem while testing the existence of a DSI.");
		} finally {
		    try {
			    if (rs != null) {
			        rs.close();
			        rs = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
					resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		return exist;
	}

	//the map of [pclassmapping->table of int]
	//build dynamically
	private HashMap associationTable =  new HashMap();
	
	public HashMap getAssociationTable(){
		return associationTable;
	}
	
	public void addAssociation(PClassMapping targetClass, int[] indexes){
		associationTable.put(targetClass, indexes);
	}
	
	public int[] getIndexesTable(PClassMapping targetClass){
		return (int[]) associationTable.get(targetClass);
	}
	

    private String updatebpelBPI10Table(
        Connection connection,
        Object conn,
        PAccessorGen paccessorGen,
        BPIEntityBeanAccessor paccessor,
    	PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
            pstmt = connection.prepareStatement(res = "UPDATE bpelBPI SET targetNamespace = ?, processName = ?, creationDate = ?, state = ?, BpelProcessBean_id = ?, id = ? WHERE id = ?");
				//targetNamespace
                nullField = paccessor.paGetTargetNamespace();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//processName
                nullField = paccessor.paGetProcessName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//creationDate
                nullField = paccessor.paGetCreationDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 3, ((java.util.Date) nullField), null);
                }
				//state
                nullField = paccessor.paGetState();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//BpelProcessBean_id
                nullField = processPnc.encodeOint(paccessor.paGetProcess(conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 5, ((java.lang.Integer) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 6, ((java.lang.Integer) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 7, ((java.lang.Integer) nullField));
                }
            } else if (paccessorGen != null) {
            pstmt = connection.prepareStatement(res = "UPDATE bpelBPI SET targetNamespace = ?, processName = ?, creationDate = ?, state = ?, BpelProcessBean_id = ?, id = ? WHERE id = ?");
				//targetNamespace
                nullField = paccessorGen.paGetStringField(targetNamespace_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//processName
                nullField = paccessorGen.paGetStringField(processName_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//creationDate
                nullField = paccessorGen.paGetDateField(creationDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 3, ((java.util.Date) nullField), null);
                }
				//state
                nullField = paccessorGen.paGetStringField(state_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//BpelProcessBean_id
                nullField = processPnc.encodeOint(paccessorGen.paGetRefField(process_NAME, conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 5, ((java.lang.Integer) nullField));
                }
				//id
                nullField = paccessorGen.paGetOintField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 6, ((java.lang.Integer) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 7, ((java.lang.Integer) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String insertbpelBPI10Table(Connection connection,
                                                Object conn,
                                                PAccessorGen paccessorGen,
                                                BPIEntityBeanAccessor paccessor,
    											PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {

		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO bpelBPI (targetNamespace, processName, creationDate, state, BpelProcessBean_id, id) VALUES (?, ?, ?, ?, ?, ?)");
				//targetNamespace
                nullField = paccessor.paGetTargetNamespace();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//processName
                nullField = paccessor.paGetProcessName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//creationDate
                nullField = paccessor.paGetCreationDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 3, ((java.util.Date) nullField), null);
                }
				//state
                nullField = paccessor.paGetState();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//BpelProcessBean_id
                nullField = processPnc.encodeOint(paccessor.paGetProcess(conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 5, ((java.lang.Integer) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 6, ((java.lang.Integer) nullField));
                }
            } else if (paccessorGen != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO bpelBPI (targetNamespace, processName, creationDate, state, BpelProcessBean_id, id) VALUES (?, ?, ?, ?, ?, ?)");
				//targetNamespace
                nullField = paccessorGen.paGetStringField(targetNamespace_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//processName
                nullField = paccessorGen.paGetStringField(processName_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//creationDate
                nullField = paccessorGen.paGetDateField(creationDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 3, ((java.util.Date) nullField), null);
                }
				//state
                nullField = paccessorGen.paGetStringField(state_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//BpelProcessBean_id
                nullField = processPnc.encodeOint(paccessorGen.paGetRefField(process_NAME, conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 5, ((java.lang.Integer) nullField));
                }
				//id
                nullField = paccessorGen.paGetOintField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 6, ((java.lang.Integer) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String deletebpelBPI10Table(
    	Connection connection,
    	Object conn,
    	PAccessorGen paccessorGen,
    	BPIEntityBeanAccessor paccessor,
    	PName pname) throws PException, SQLException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            pstmt = connection.prepareStatement(res = "DELETE FROM bpelBPI WHERE id = ?");
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 1, ((java.lang.Integer) nullField));
                }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }

	/**
	 * Is the RdbAdapter to use for managing the various relational database.
	 * This field must not be null after the mapping of the class
	 */
	public RdbAdapter adapter;
	/**
	 * Is the select query to use in the Binding.read method
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The read query is built once time
	 * when the class is mapped.
	 */
	public String readQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String prefetchedExtentQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is NOT activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String extentQuery = null;

	/**
	 * Building of the structure defining the FROM clause with
	 *  JoinedTable instances
	 */
	private JoinedTable getJoinedTable() {
		JoinedTable jt = new JoinedTable("bpelBPI");
		JoinedTable.Join jtj = null;
        return jt;
	}

    public String[] getPrefetchSelectCols() {
		return new String[]{"bpelBPI.BpelProcessBean_id", "bpelBPI.creationDate", "bpelBPI.id", "bpelBPI.processName", "bpelBPI.state", "bpelBPI.targetNamespace"};
	}

	public String getPrefetchWhereClause() {
		return null;
	}

	public java.util.Properties getCNId2Column() {
		java.util.Properties p = new java.util.Properties();
		p.setProperty("", "bpelBPI.id");
		return p;
	}

	public List getPrefetchTables() {
		return Collections.singletonList(getJoinedTable());
	}

	/**
	 * Compute the read query with the help of the RdbAdapter
	 */
	public void computeReadQuery() {
		readQuery = adapter.getQuery("bpelBPI.BpelProcessBean_id, bpelBPI.creationDate, bpelBPI.id, bpelBPI.processName, bpelBPI.state, bpelBPI.targetNamespace",
			Collections.singletonList(getJoinedTable()),
			"bpelBPI.id = ?",
			false, false);
	}

	/**
	 * Compute the extent query (without prefetch) with the help of the
	 * RdbAdapter
	 */
	public void computeExtentQuery() throws ExpressionException {
		if (extentQuery == null) {
			extentQuery = adapter.getQuery("bpelBPI.id",
				Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}
	}

	/**
	 * Compute the extent query (with prefetch) with the help of the RdbAdapter
	 */
	public void computePrefetchedExtentQuery() {
		if (prefetchedExtentQuery == null) {
			prefetchedExtentQuery = adapter.getQuery("bpelBPI.BpelProcessBean_id"+ adapter.getColumnAliasExpr("process_id") + ", bpelBPI.creationDate"+ adapter.getColumnAliasExpr("creationDate") + ", bpelBPI.id"+ adapter.getColumnAliasExpr("id") + ", bpelBPI.processName"+ adapter.getColumnAliasExpr("processName") + ", bpelBPI.state"+ adapter.getColumnAliasExpr("state") + ", bpelBPI.targetNamespace"+ adapter.getColumnAliasExpr("targetNamespace") + ", bpelBPI.id" + adapter.getColumnAliasExpr("pk0") + "",
			Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}

	}


	public PName decode(ResultSet rs, int startindex, Object png) throws IOException, SQLException, PException {
        return classPNC.decodeOint(adapter.getOint(rs, startindex, null));
	}



	
	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
		((Loggable) bPUsGCM).setLogger(l);
		((Loggable) messagesGCM).setLogger(l);
		((Loggable) partnerLinksGCM).setLogger(l);
		((Loggable) scopesGCM).setLogger(l);
		((Loggable) variablesGCM).setLogger(l);
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
}




class BPIEntityBeanResultIterator implements PNameIterator {
    ResultSet rs;
	private PBinder binder;
	private BPIEntityBeanMapping pclassMapping;
    private boolean hasNext;
    private boolean isForw;
	private PreparedStatement ps;
	private Logger logger;

    private boolean prefetching;
    
    private PPrefetchTupleCollection tc = null;
    private int pnameIndex;

	BPIEntityBeanResultIterator(Object txctx,
		PreparedStatement ps,
		ResultSet r,
		PBinder binder,
		boolean prefetching,
		boolean withSubType,
		Logger l,
		int pi,
		BPIEntityBeanMapping pcm) throws SQLException, MedorException {

		this.ps = ps;
		this.binder = binder;
		this.prefetching = prefetching;
        rs = r;
        logger = l;
        pclassMapping = pcm;
        if (!prefetching) {
        	pnameIndex = 1;
        } else {
        	pnameIndex = pi;
        	tc = new RdbTupleCollection(txctx, r, ps, pnameIndex, logger, pclassMapping, withSubType, this);
        }
        hasNext = (!prefetching) ? rs.next() : tc.next();
        isForw = true;
    }

	public void finalize() throws Throwable {
		if (rs != null)
			rs.close();
		if (ps != null)
			ps.close();
		super.finalize();
	}

    public void close() {
        if (!prefetching) {
            if (rs!= null) {
                try {
                    rs.close();
                } catch(Exception e) {
                }
                rs = null;
            }
            hasNext = false;
            isForw = false;
        }
    }

    public boolean hasNext() {
    	boolean res;
        if (isForw) {
            res = hasNext;
        } else {
            isForw = true;
            try {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                res = hasNext;
            } catch(Exception e) {
                e.printStackTrace();
                res = false;
            }
        }
        if (res == false) {
        	try {
				if (rs != null) {
					rs.close();
					rs = null;
				}
				if (ps != null) {
					ps.close();
					ps = null;
				}
			} catch (SQLException e) {
				rs = null;
				ps = null;
			}
        }
        return res;
    }

    public void remove() throws UnsupportedOperationException, IllegalStateException {
        throw new UnsupportedOperationException();
    }

    public Object next() throws NoSuchElementException {
        try {
            if (!isForw) {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                isForw = true;
            }
            if (!hasNext) {
                throw new NoSuchElementException();
            }
            isForw = false;
            if (prefetching) {
            	tc.getPrefetchBuffer().addPrefetchTuple();
            }
            return pclassMapping.decode(rs, pnameIndex, null);
        } catch (NoSuchElementException e) {
        	throw e;
        } catch (Exception e) {
        	if (logger != null) {
        		logger.log(BasicLevel.ERROR, "Impossible to fetch the next element", e);
        	}
            throw new NoSuchElementException(
                    "Impossible to decode to an ID PName: " + e.getMessage());
        }
    }
}
