/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package jorm.rdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PClassMappingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PMappingStructuresManager;
import org.objectweb.jorm.api.PNameIterator;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.genclass.api.FieldDesc;
import org.objectweb.jorm.genclass.api.GenClassProp;
import org.objectweb.jorm.lib.EmptyPNameIterator;
import org.objectweb.jorm.lib.MultiplePNameIterator;
import org.objectweb.medor.eval.lib.TupleAdapter;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
//import for the rdb mapper
import org.objectweb.jorm.api.PPrefetchTupleCollection;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import org.objectweb.jorm.mapper.rdb.adapter.api.JoinedTable;
import org.objectweb.jorm.mapper.rdb.lib.RdbExtentGenInfos;
import org.objectweb.jorm.mapper.rdb.lib.RdbTupleCollection;
import org.objectweb.jorm.mapper.rdb.lib.PClassMappingDecoder;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;
import org.objectweb.jorm.mapper.rdb.lib.RdbPMappingStructuresManager;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbClassMapping;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbMapping;
import org.objectweb.jorm.mapper.rdb.lib.RdbPPolymorphicClass;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.lib.SelectEvaluator;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.eval.api.ConnectionResources;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.CalculatedParameterOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.query.rdb.lib.BasicRdbStringQueryLeaf;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.datasource.lib.ConnectionFactoryDataStore;
import org.objectweb.medor.expression.lib.BasicBinaryArithmeticOperator;
import org.objectweb.medor.expression.lib.Plus;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.eval.prefetch.api.IntermediaryPrefetchBuffer;
import org.objectweb.medor.eval.prefetch.lib.IntermediaryPrefetchBufferImpl;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.jorm.lib.EncodePName;
import org.objectweb.medor.filter.jorm.lib.CompositePName;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.optim.lib.IndexesGenerator;
import org.objectweb.perseus.persistence.api.ConnectionHolder;
import org.objectweb.perseus.persistence.api.WorkingSet;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Collections;
import java.util.Collection;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.mapper.rdb.lib.RdbPrefetchablePCM;
// There are Serialized fields
import org.objectweb.jorm.mapper.rdb.lib.RdbBlob;
import java.io.ObjectInputStream;
import java.io.Serializable;


// Import clauses required by the object MessageBean
import jorm.MessageBeanAccessor;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

public class MessageBeanMapping
	extends org.objectweb.jonas_ejb.container.jorm.RdbFactory
	implements PClassMapping, PClassMappingCtrl, Loggable
		, PClassMappingDecoder
		, RdbPrefetchablePCM
    {
	protected static final String MAPPER_NAME = "rdb";
	protected static final String PROJECT_NAME = "jonas";
	//Fields
	protected static final String bPI_NAME = "bPI";
	protected static final String date_NAME = "date";
	protected static final String fault_NAME = "fault";
	protected static final String faultMessage_NAME = "faultMessage";
	protected static final String faultName_NAME = "faultName";
	protected static final String id_NAME = "id";
	protected static final String namespace_NAME = "namespace";
	protected static final String operation_NAME = "operation";
	protected static final String partner_NAME = "partner";
	protected static final String portType_NAME = "portType";
	protected static final String processName_NAME = "processName";
	protected static final String refPartner_NAME = "refPartner";
	protected static final String state_NAME = "state";
	protected static final String statementNb_NAME = "statementNb";
	protected static final String targetNamespace_NAME = "targetNamespace";
	protected static final String type_NAME = "type";
	protected static final String value_NAME = "value";
	//Hidden fields
	protected static final String bPI_id_NAME = "bPI_id";

	private final static String[][] SUPER_TYPES =  {};

	/** Fully quailied name of the persitent class */
	protected final static String JORM_CLASS_NAME =
		"jorm.MessageBean";
	/** The actual binder which does the binding action */
	private PBinder binder;
	private PNameCoder classPNC;

	/** The meta object of the current jorm class */
	private Class moc;
	/** The mapper linked to this PClassMapping */
	private PMapper mapper;
	public PNameCoder bPIPnc;
	/** the PClassMapping of the sub type */
	private PClassMapping[] subPCMs = null;

	//variables for the resolve method
	private int beginIndex = 0;
	private QueryTree queryTree = null;
	private FieldOperand[] fieldOperands = null;
	private String[] compositeFieldNames = null;
 	/**
	 * The logger instance used by this XMapping instance
	 * and the XBinding instances
	 */
	public Logger logger;
	public LoggerFactory loggerFactory;

	public MessageBeanMapping() {
 	}
	public PNameCoder getBPIPNameCoder() {
	    return bPIPnc;
	}

	// IMPLEMENTATION OF METHODS FROM THE PClassMappingCtrl INTERFACE

    public void setPNameCoder(PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setPNameCoder(String fn, PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		if (bPI_NAME.equals(fn)) {
			bPIPnc = (PNamingContext) pnc;
			return;
		}
		throw new UnsupportedOperationException("This class does'nt contain field reference: " + fn);
    }

    public void setGenClassMapping(PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setGenClassMapping(String fn, PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class does not contain references to generic class");
    }

    public void setProjectName(String projectname) {
    }


	// IMPLEMENTATION OF METHODS FROM THE PClassMapping INTERFACE

	public void init(PMappingCallback m, MetaObject metaclass) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		mapper = m;
		moc = (Class) metaclass;
		//the getPType method inits the ptype if not already done
		PType ptype = mapper.getPTypeSpace().createPType(
				"jorm.MessageBean",
				SUPER_TYPES);
		setPType(ptype);
        // PType Assignement to the PNC managing the reference bPI
        String[][] bPIICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            "jorm.BPIEntityBean",
            bPIICDef);
        bPIPnc.setPType(ptype);
        
         try {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}
 		//register the structures and dependencies
		init(mapper.getPMappingStructuresManager());
        } catch (Exception e) {
			if(e instanceof RdbAdapterException)
				throw new PException(e, "Column type computing error: ");
			else
				e.printStackTrace();
		} 
			computeReadQuery();
		try {
			computeExtentQuery();
		} catch (ExpressionException e) {
			throw new PException(e);
		}
		computePrefetchedExtentQuery();
		}

	public void init(PMappingStructuresManager pmsm) throws PException {
		pmsm.declareClass(JORM_CLASS_NAME);
		try {
        RdbPMappingStructuresManager msm = (RdbPMappingStructuresManager) pmsm;
        if (mapper == null) {
	       	mapper = msm.getPMapper();
	    }
	    if (adapter == null) {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}
		}
	    // Initialization of the structures of the class jorm.MessageBean
			        	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "BPIEntityBean_id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "namespace", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "value", adapter.getSqlType(PTypeSpace.SERIALIZED.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "type", adapter.getSqlType(PTypeSpace.SERIALIZED.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "fault", adapter.getSqlType(PTypeSpace.BOOLEAN.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "processName", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "id", adapter.getSqlType(PTypeSpace.OBJINT.getTypeCode(), true, -1, -1), true, true, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "operation", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "targetNamespace", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "partner", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "faultMessage", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "state", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "faultName", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "portType", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "refPartner", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "statementNb", adapter.getSqlType(PTypeSpace.INT.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "bpelMessage", "date_", adapter.getSqlType(PTypeSpace.LONG.getTypeCode(), false, -1, -1), false, false, true);
	    					//dependencies
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BPIEntityBean");
		} catch(Exception e) {
			throw new PException("Error while registering mapping structures: " + e.getMessage());
		}
		classDefined(pmsm);
	}
	
	protected void classDefined() throws PException {
		//register class in the cluster
        classDefined(mapper.getPMappingStructuresManager());
	}

	public void classDefined(PMappingStructuresManager pmsm) throws PException {
		if (pmsm == null) {
			throw new PException("PMappingStructureManager is null: " + JORM_CLASS_NAME + " cannot be registered in a cluster.");
		}
		//register class in the cluster
        pmsm.classDefined(JORM_CLASS_NAME);
	}
	
	public PMapper getPMapper() {
		return mapper;
	}

	public PBinder getPBinder() {
		return binder;
	}

	public PNameCoder getClassPNameCoder() {
		return classPNC;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains references*/
	public PNameCoder getPNameCoder()
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PNameCoder getPNameCoder(String fn)
		throws UnsupportedOperationException {
			if (bPI_NAME.equals(fn))
				return bPIPnc;
		throw new UnsupportedOperationException("This class does'nt contain field reference " + fn);
	}

	public String getClassName() {
		return JORM_CLASS_NAME;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains another gen class*/
	public PClassMapping getGenClassMapping() throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PClassMapping getGenClassMapping(String fn)
		throws UnsupportedOperationException {
			throw new UnsupportedOperationException("This class does not contain references to generic class");
	}

	public boolean isConform(String mapperName) {
		return mapperName.startsWith(MAPPER_NAME);
	}

	public String getProjectName() {
		return PROJECT_NAME;
	}

	public MetaObject getMetaInfo() {
		return moc;
	}

	public String[][] getSuperClassNames() {
		return SUPER_TYPES;
	}

	public void setPBinder(PBinder b) {
		binder = b;
	}

	public void setClassPNameCoder(PNameCoder pnc) throws PException {
		classPNC = pnc;
	}

	public PBinding createPBinding() throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		PBinding binding = new jorm.MessageBeanBinding();
		binding.init(this);
		return binding;
	}
	/** @deprecated */
    public Iterator getPNameIterator(Object conn) throws PException {
		return getPNameIterator(conn, false, false, null);
    }

	public PNameIterator getPNameIterator(Object conn,
	                                      boolean withSubType,
	                                      boolean prefetching,
	                                      Object txctx) throws PException {
		PNameIterator it = null;
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String mapperName = mapper.getMapperName();
			if(mapperName.indexOf(".") != -1)
				mapperName = mapperName.substring(0, mapperName.indexOf("."));	
			RdbMapping rdbm = (RdbMapping) mapper.getMetaInfoManager().getClass(this.getClassName()).getClassProject(this.getProjectName()).getMapping(mapperName);
			RdbClassMapping rdbcm = (RdbClassMapping) rdbm.getClassMapping();
			String queryToExecute = "";
			int beginIndex = 0;
				queryToExecute = (prefetching ? prefetchedExtentQuery : extentQuery);
				//System.out.println("query to execute:" + queryToExecute);
				//compute the begining index for the PName: number of fields to select + 1
				beginIndex = 17 + 1;
			PreparedStatement ps = connection.prepareStatement(queryToExecute);
			it = new MessageBeanResultIterator(txctx, ps, ps.executeQuery(), binder, prefetching, withSubType, logger, beginIndex, this);
        } catch (SQLException e) {
            throw new PExceptionIO(e, "SQL problem while computing the PName Iterator: class <MessageBean>.");
        } catch (MedorException e) {
            throw new PExceptionIO(e, "Medor problem while initializing the PName Iterator: class <MessageBean>.");
        }
		return it;
	}

	public PClassMapping[] getSubPCMs() throws PException {
		PType ptype = getPType();
		if (ptype == null) {
			throw new PException("No PType defined for the PClassMapping of the class" + JORM_CLASS_NAME);
		}
		PType[] subTypes = ptype.getSubTypes();
		if (subTypes == null || subTypes.length == 0) {
			//No direct subType are mapped then return null
			return null;
		}
		if (subPCMs == null || subPCMs.length != subTypes.length) {
			//compute the PCM array
			subPCMs = new PClassMapping[subTypes.length];
			for (int i=0; i<subTypes.length; i++) {
				subPCMs[i] = mapper.lookup(subTypes[i].getJormName());
			}
		}
		return subPCMs;
	}
	
	// ADD YO //
	public synchronized PName resolve(Object conn, PName pname) throws PException {
				//mapper rdb but not polymorphic
		return pname;
	 	}
	
	public boolean match(Object obj, boolean intermediaryTuple) throws PException {
		return false;
	}
	
	public PName getDecodedPName(TupleCollection tc, PName pname, boolean intermediaryTuple) throws PException {
		PBinder binder = null;
	//test if the tuple is an instance of this class
	if (this.match(tc, intermediaryTuple)) {
		binder  = this.getPBinder();
	} else {
		//test for all the children
		boolean match = false;
		for (int i = 0; i<subPCMs.length && !match; i++) {
				match = subPCMs[i].match(tc, intermediaryTuple);
				binder = subPCMs[i].getPBinder();
		}
		//if no pcm found, error
		if (!match)
			throw new PException("Problem with Medor while resolving pname on the PCM " + JORM_CLASS_NAME + ". No class found for the pname " + pname.toString() + ".");
	}
			return binder.decodeOint(pname.encodeOint());
		}
	// END ADD YO //
	
    public void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException {
		bPIPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bPI_NAME, "jorm.BPIEntityBean");
    }

	public PType getPType() {
		return  binder.getPType();
	}

	public void setPType(PType pt) {
		binder.setPType(pt);
	}


	public void write(PBinding pb, Object conn, PAccessor pa)
		throws PException {
		MessageBeanAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof MessageBeanAccessor)
			paccessor = (MessageBeanAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized WRITE operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String sqlquery;
		if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Remove");
//Table bpelMessage : readOnly=false mustGenerateUpdate=true colocatedTable=false joins=$tab.joins
                sqlquery = deletebpelMessage7Table(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [MessageBean]: problematic SQL query -> {" + sqlquery + "}");
                }
		} else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Insert");
                sqlquery = insertbpelMessage7Table(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [MessageBean]: problematic SQL query -> {" + sqlquery + "}");
                }
		} else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Update");
                sqlquery = updatebpelMessage7Table(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [MessageBean]: problematic SQL query -> {" + sqlquery + "}");
                }
		}
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se, "SQL problem while writing the DSI.");
		} catch (java.io.IOException ioe) {
			throw new PExceptionIO(ioe, "Stream IO problem while writing the DSI.");
		} catch (RdbAdapterException rae) {
			throw new PExceptionIO(rae, "SQL problem while writing the DSI.");
		}
		// Changing the current state
		((PBindingCtrl) pb).setStatus(nextState);
	}
	public void read(PBinding pb, Object conn, PAccessor pa) throws PException {
		read(pb, conn, pa, false);
	}
	
	public void read(PBinding pb, Object conn, PAccessor pa, boolean forUpdate) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Read");
		byte status = pb.getStatus();
		MessageBeanAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof MessageBeanAccessor)
			paccessor = (MessageBeanAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet resultSet = null;
        PException resex = null;
		try {
            String jormrq = readQuery;
            if (forUpdate) {
            	jormrq = jormrq + " FOR UPDATE";
            }
            pstmt = connection.prepareStatement(jormrq);
            adapter.setOint(pstmt, 1, pname.encodeOint());
            resultSet = pstmt.executeQuery();
            if(resultSet.next() == false) {
                throw new PExceptionNoDSI(
                    "No entry matching with the following identifier: " 
                    + pname + ", query=" + jormrq);
            }
            PName refname;
            Object fieldval;
            long notnullval;
			if (paccessor != null) {
				//read public primitive fields
                paccessor.paSetDate(adapter.getLong(resultSet, 2, ((long) -1)));
                paccessor.paSetFault(adapter.getBoolean(resultSet, 3, false));
                paccessor.paSetFaultMessage(adapter.getString(resultSet, 4, null));
                paccessor.paSetFaultName(adapter.getString(resultSet, 5, null));
                paccessor.paSetId(adapter.getOint(resultSet, 6, null));
                paccessor.paSetNamespace(adapter.getString(resultSet, 7, null));
                paccessor.paSetOperation(adapter.getString(resultSet, 8, null));
                paccessor.paSetPartner(adapter.getString(resultSet, 9, null));
                paccessor.paSetPortType(adapter.getString(resultSet, 10, null));
                paccessor.paSetProcessName(adapter.getString(resultSet, 11, null));
                paccessor.paSetRefPartner(adapter.getString(resultSet, 12, null));
                paccessor.paSetState(adapter.getString(resultSet, 13, null));
                paccessor.paSetStatementNb(adapter.getInt(resultSet, 14, ((int) -1)));
                paccessor.paSetTargetNamespace(adapter.getString(resultSet, 15, null));
                paccessor.paSetType(adapter.getSerialized(resultSet, 16, null));
                paccessor.paSetValue(adapter.getSerialized(resultSet, 17, null));
				//decode PName for references
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@15e6691 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@9fe666 / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@1559ba8 / fieldName=bPI / join=null
				//fieldName=bPI_id
                fieldval = adapter.getOint(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = bPIPnc.decodeOint(null);
                } else {
                    refname = bPIPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessor.paSetBPI(refname, conn);
			} else if (paccessorGen != null) {
				//read public primitive fields
                paccessorGen.paSetLongField(date_NAME, adapter.getLong(resultSet, 2, ((long) -1)));
                paccessorGen.paSetBooleanField(fault_NAME, adapter.getBoolean(resultSet, 3, false));
                paccessorGen.paSetStringField(faultMessage_NAME, adapter.getString(resultSet, 4, null));
                paccessorGen.paSetStringField(faultName_NAME, adapter.getString(resultSet, 5, null));
                paccessorGen.paSetOintField(id_NAME, adapter.getOint(resultSet, 6, null));
                paccessorGen.paSetStringField(namespace_NAME, adapter.getString(resultSet, 7, null));
                paccessorGen.paSetStringField(operation_NAME, adapter.getString(resultSet, 8, null));
                paccessorGen.paSetStringField(partner_NAME, adapter.getString(resultSet, 9, null));
                paccessorGen.paSetStringField(portType_NAME, adapter.getString(resultSet, 10, null));
                paccessorGen.paSetStringField(processName_NAME, adapter.getString(resultSet, 11, null));
                paccessorGen.paSetStringField(refPartner_NAME, adapter.getString(resultSet, 12, null));
                paccessorGen.paSetStringField(state_NAME, adapter.getString(resultSet, 13, null));
                paccessorGen.paSetIntField(statementNb_NAME, adapter.getInt(resultSet, 14, ((int) -1)));
                paccessorGen.paSetStringField(targetNamespace_NAME, adapter.getString(resultSet, 15, null));
                paccessorGen.paSetSerializedField(type_NAME, adapter.getSerialized(resultSet, 16, null));
                paccessorGen.paSetSerializedField(value_NAME, adapter.getSerialized(resultSet, 17, null));
				//decode PName for references
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@15e6691 / cnId=-1 / cnPackage=null / cnFieldColumns=null / refColumn=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@9fe666 / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@1559ba8 / fieldName=bPI / join=null
				//fieldName=bPI_id
                fieldval = adapter.getOint(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = bPIPnc.decodeOint(null);
                } else {
                    refname = bPIPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessorGen.paSetRefField(bPI_NAME, refname, conn);
			}
		} catch (PException pe) {
			resex = pe;
		} catch (Exception pe) {
			resex = new PExceptionIO(pe, "Problem while reading the DSI.");
		} finally {
		    try {
		        if (resultSet != null) {
			        resultSet.close();
			        resultSet = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
			    	resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		((PBindingCtrl) pb).setStatus(nextState);
	}
	public void read(PBinding pb, Object conn, PAccessor pa, Object txctx) throws PException {
		read(pb, conn, pa, txctx, false);
	}

	public void read(PBinding pb, Object conn, PAccessor pa, Object txctx, boolean forUpdate) throws PException {
	    PrefetchCache pc = getPMapper().getPrefetchCache();
	    if (pc == null || txctx == null) {
	        read(pb, conn, pa, forUpdate);
	        return;
	    }
		PName pname = pb.getPName();
	    try {
	        Tuple tuple = pc.getPrefetchTuple(pname, this, txctx);
	        if (tuple == null) {
	            read(pb, conn, pa, forUpdate);
	            return;
	        }
		    MessageBeanAccessor paccessor = null;
		    PAccessorGen paccessorGen = null;
		    if (pa == null)
			    throw new PExceptionProtocol("Accessor must not be null");
		    else if (pa instanceof MessageBeanAccessor)
			    paccessor = (MessageBeanAccessor) pa;
		    else if (pa instanceof PAccessorGen)
			    paccessorGen = (PAccessorGen) pa;
		    else
			    throw new PExceptionProtocol("Wrong type of accessor: " + pa);
			byte status = pb.getStatus();
		    byte nextState = PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		    if (nextState==PBinding.LIFECYCLE_ERROR)
			    throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
            Object fval;
            long notnullval;
            if (paccessor != null) {
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 1, PTypeSpace.OBJINT));
                if (tuple.isDefined(1)) {
                    paccessor.paSetBPI(bPIPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessor.paSetBPI(bPIPnc.getNull(), conn);
                }
                paccessor.paSetDate(tuple.getLong(2));
                paccessor.paSetFault(tuple.getBoolean(3));
                paccessor.paSetFaultMessage(tuple.getString(4));
                paccessor.paSetFaultName(tuple.getString(5));
                paccessor.paSetId(((java.lang.Integer) TupleAdapter.getObject(tuple, 6, PTypeSpace.OBJINT)));
                paccessor.paSetNamespace(tuple.getString(7));
                paccessor.paSetOperation(tuple.getString(8));
                paccessor.paSetPartner(tuple.getString(9));
                paccessor.paSetPortType(tuple.getString(10));
                paccessor.paSetProcessName(tuple.getString(11));
                paccessor.paSetRefPartner(tuple.getString(12));
                paccessor.paSetState(tuple.getString(13));
                paccessor.paSetStatementNb(tuple.getInt(14));
                paccessor.paSetTargetNamespace(tuple.getString(15));
                paccessor.paSetType((Serializable) tuple.getObject(16));
                paccessor.paSetValue((Serializable) tuple.getObject(17));
            } else if (paccessorGen != null) {
				fval =
					((java.lang.Integer) TupleAdapter.getObject(tuple, 1, PTypeSpace.OBJINT));
                if (tuple.isDefined(1)) {
                    paccessorGen.paSetRefField(bPI_NAME, bPIPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessorGen.paSetRefField(bPI_NAME, bPIPnc.getNull(), conn);
                }
                paccessorGen.paSetLongField(date_NAME, tuple.getLong(2));
                paccessorGen.paSetBooleanField(fault_NAME, tuple.getBoolean(3));
                paccessorGen.paSetStringField(faultMessage_NAME, tuple.getString(4));
                paccessorGen.paSetStringField(faultName_NAME, tuple.getString(5));
                paccessorGen.paSetOintField(id_NAME, ((java.lang.Integer) TupleAdapter.getObject(tuple, 6, PTypeSpace.OBJINT)));
                paccessorGen.paSetStringField(namespace_NAME, tuple.getString(7));
                paccessorGen.paSetStringField(operation_NAME, tuple.getString(8));
                paccessorGen.paSetStringField(partner_NAME, tuple.getString(9));
                paccessorGen.paSetStringField(portType_NAME, tuple.getString(10));
                paccessorGen.paSetStringField(processName_NAME, tuple.getString(11));
                paccessorGen.paSetStringField(refPartner_NAME, tuple.getString(12));
                paccessorGen.paSetStringField(state_NAME, tuple.getString(13));
                paccessorGen.paSetIntField(statementNb_NAME, tuple.getInt(14));
                paccessorGen.paSetStringField(targetNamespace_NAME, tuple.getString(15));
                paccessorGen.paSetSerializedField(type_NAME, (Serializable) tuple.getObject(16));
                paccessorGen.paSetSerializedField(value_NAME, (Serializable) tuple.getObject(17));
            }
			((PBindingCtrl) pb).setStatus(nextState);
		} catch (MedorException me) {
		    throw new PExceptionProtocol(me, "Problem while trying to prefetch data from Medor");
		}
    }

	public boolean exist(PBinding pb, Object conn) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Exist");
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized EXIST operation, status=" + status);
		((PBindingCtrl) pb).setStatus(nextState);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        PException resex = null;
        boolean exist = false;
		try {
            pstmt = connection.prepareStatement("SELECT bpelMessage.id FROM bpelMessage WHERE bpelMessage.id = ?");
            adapter.setOint(pstmt, 1, pname.encodeOint());
			rs = pstmt.executeQuery();
            exist = rs.next();
		} catch (java.sql.SQLException se) {
			resex = new PExceptionIO(se, "SQL problem while testing the existence of a DSI.");
		} finally {
		    try {
			    if (rs != null) {
			        rs.close();
			        rs = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
					resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		return exist;
	}

	//the map of [pclassmapping->table of int]
	//build dynamically
	private HashMap associationTable =  new HashMap();
	
	public HashMap getAssociationTable(){
		return associationTable;
	}
	
	public void addAssociation(PClassMapping targetClass, int[] indexes){
		associationTable.put(targetClass, indexes);
	}
	
	public int[] getIndexesTable(PClassMapping targetClass){
		return (int[]) associationTable.get(targetClass);
	}
	

    private String updatebpelMessage7Table(
        Connection connection,
        Object conn,
        PAccessorGen paccessorGen,
        MessageBeanAccessor paccessor,
    	PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {
		java.sql.Blob blob = null;
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
            pstmt = connection.prepareStatement(res = "UPDATE bpelMessage SET BPIEntityBean_id = ?, namespace = ?, value = ?, type = ?, fault = ?, processName = ?, id = ?, operation = ?, targetNamespace = ?, partner = ?, faultMessage = ?, state = ?, faultName = ?, portType = ?, refPartner = ?, statementNb = ?, date_ = ? WHERE id = ?");
				//BPIEntityBean_id
                nullField = bPIPnc.encodeOint(paccessor.paGetBPI(conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 1, ((java.lang.Integer) nullField));
                }
				//namespace
                nullField = paccessor.paGetNamespace();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//value
                nullField = paccessor.paGetValue();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 3, ((java.io.Serializable) nullField));
                }
				//type
                nullField = paccessor.paGetType();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 4, ((java.io.Serializable) nullField));
                }
				//fault
				adapter.setBoolean(pstmt, 5, paccessor.paGetFault());
				//processName
                nullField = paccessor.paGetProcessName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 7, ((java.lang.Integer) nullField));
                }
				//operation
                nullField = paccessor.paGetOperation();
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
				//targetNamespace
                nullField = paccessor.paGetTargetNamespace();
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 9, ((java.lang.String) nullField));
                }
				//partner
                nullField = paccessor.paGetPartner();
                if (nullField == null) {
                    adapter.setNull(pstmt, 10, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 10, ((java.lang.String) nullField));
                }
				//faultMessage
                nullField = paccessor.paGetFaultMessage();
                if (nullField == null) {
                    adapter.setNull(pstmt, 11, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 11, ((java.lang.String) nullField));
                }
				//state
                nullField = paccessor.paGetState();
                if (nullField == null) {
                    adapter.setNull(pstmt, 12, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 12, ((java.lang.String) nullField));
                }
				//faultName
                nullField = paccessor.paGetFaultName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 13, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 13, ((java.lang.String) nullField));
                }
				//portType
                nullField = paccessor.paGetPortType();
                if (nullField == null) {
                    adapter.setNull(pstmt, 14, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 14, ((java.lang.String) nullField));
                }
				//refPartner
                nullField = paccessor.paGetRefPartner();
                if (nullField == null) {
                    adapter.setNull(pstmt, 15, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 15, ((java.lang.String) nullField));
                }
				//statementNb
				adapter.setInt(pstmt, 16, paccessor.paGetStatementNb());
				//date_
				adapter.setLong(pstmt, 17, paccessor.paGetDate());
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 18, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 18, ((java.lang.Integer) nullField));
                }
            } else if (paccessorGen != null) {
            pstmt = connection.prepareStatement(res = "UPDATE bpelMessage SET BPIEntityBean_id = ?, namespace = ?, value = ?, type = ?, fault = ?, processName = ?, id = ?, operation = ?, targetNamespace = ?, partner = ?, faultMessage = ?, state = ?, faultName = ?, portType = ?, refPartner = ?, statementNb = ?, date_ = ? WHERE id = ?");
				//BPIEntityBean_id
                nullField = bPIPnc.encodeOint(paccessorGen.paGetRefField(bPI_NAME, conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 1, ((java.lang.Integer) nullField));
                }
				//namespace
                nullField = paccessorGen.paGetStringField(namespace_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//value
                nullField = paccessorGen.paGetSerializedField(value_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 3, ((java.io.Serializable) nullField));
                }
				//type
                nullField = paccessorGen.paGetSerializedField(type_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 4, ((java.io.Serializable) nullField));
                }
				//fault
				adapter.setBoolean(pstmt, 5, paccessorGen.paGetBooleanField(fault_NAME));
				//processName
                nullField = paccessorGen.paGetStringField(processName_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessorGen.paGetOintField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 7, ((java.lang.Integer) nullField));
                }
				//operation
                nullField = paccessorGen.paGetStringField(operation_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
				//targetNamespace
                nullField = paccessorGen.paGetStringField(targetNamespace_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 9, ((java.lang.String) nullField));
                }
				//partner
                nullField = paccessorGen.paGetStringField(partner_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 10, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 10, ((java.lang.String) nullField));
                }
				//faultMessage
                nullField = paccessorGen.paGetStringField(faultMessage_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 11, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 11, ((java.lang.String) nullField));
                }
				//state
                nullField = paccessorGen.paGetStringField(state_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 12, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 12, ((java.lang.String) nullField));
                }
				//faultName
                nullField = paccessorGen.paGetStringField(faultName_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 13, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 13, ((java.lang.String) nullField));
                }
				//portType
                nullField = paccessorGen.paGetStringField(portType_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 14, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 14, ((java.lang.String) nullField));
                }
				//refPartner
                nullField = paccessorGen.paGetStringField(refPartner_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 15, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 15, ((java.lang.String) nullField));
                }
				//statementNb
				adapter.setInt(pstmt, 16, paccessorGen.paGetIntField(statementNb_NAME));
				//date_
				adapter.setLong(pstmt, 17, paccessorGen.paGetLongField(date_NAME));
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 18, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 18, ((java.lang.Integer) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String insertbpelMessage7Table(Connection connection,
                                                Object conn,
                                                PAccessorGen paccessorGen,
                                                MessageBeanAccessor paccessor,
    											PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {

		Object nullField = null;
		java.sql.Blob blob = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO bpelMessage (BPIEntityBean_id, namespace, value, type, fault, processName, id, operation, targetNamespace, partner, faultMessage, state, faultName, portType, refPartner, statementNb, date_) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
				//BPIEntityBean_id
                nullField = bPIPnc.encodeOint(paccessor.paGetBPI(conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 1, ((java.lang.Integer) nullField));
                }
				//namespace
                nullField = paccessor.paGetNamespace();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//value
                nullField = paccessor.paGetValue();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 3, ((java.io.Serializable) nullField));
                }
				//type
                nullField = paccessor.paGetType();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 4, ((java.io.Serializable) nullField));
                }
				//fault
				adapter.setBoolean(pstmt, 5, paccessor.paGetFault());
				//processName
                nullField = paccessor.paGetProcessName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 7, ((java.lang.Integer) nullField));
                }
				//operation
                nullField = paccessor.paGetOperation();
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
				//targetNamespace
                nullField = paccessor.paGetTargetNamespace();
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 9, ((java.lang.String) nullField));
                }
				//partner
                nullField = paccessor.paGetPartner();
                if (nullField == null) {
                    adapter.setNull(pstmt, 10, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 10, ((java.lang.String) nullField));
                }
				//faultMessage
                nullField = paccessor.paGetFaultMessage();
                if (nullField == null) {
                    adapter.setNull(pstmt, 11, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 11, ((java.lang.String) nullField));
                }
				//state
                nullField = paccessor.paGetState();
                if (nullField == null) {
                    adapter.setNull(pstmt, 12, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 12, ((java.lang.String) nullField));
                }
				//faultName
                nullField = paccessor.paGetFaultName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 13, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 13, ((java.lang.String) nullField));
                }
				//portType
                nullField = paccessor.paGetPortType();
                if (nullField == null) {
                    adapter.setNull(pstmt, 14, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 14, ((java.lang.String) nullField));
                }
				//refPartner
                nullField = paccessor.paGetRefPartner();
                if (nullField == null) {
                    adapter.setNull(pstmt, 15, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 15, ((java.lang.String) nullField));
                }
				//statementNb
				adapter.setInt(pstmt, 16, paccessor.paGetStatementNb());
				//date_
				adapter.setLong(pstmt, 17, paccessor.paGetDate());
            } else if (paccessorGen != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO bpelMessage (BPIEntityBean_id, namespace, value, type, fault, processName, id, operation, targetNamespace, partner, faultMessage, state, faultName, portType, refPartner, statementNb, date_) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
				//BPIEntityBean_id
                nullField = bPIPnc.encodeOint(paccessorGen.paGetRefField(bPI_NAME, conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 1, ((java.lang.Integer) nullField));
                }
				//namespace
                nullField = paccessorGen.paGetStringField(namespace_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//value
                nullField = paccessorGen.paGetSerializedField(value_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 3, ((java.io.Serializable) nullField));
                }
				//type
                nullField = paccessorGen.paGetSerializedField(type_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 4, ((java.io.Serializable) nullField));
                }
				//fault
				adapter.setBoolean(pstmt, 5, paccessorGen.paGetBooleanField(fault_NAME));
				//processName
                nullField = paccessorGen.paGetStringField(processName_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessorGen.paGetOintField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 7, ((java.lang.Integer) nullField));
                }
				//operation
                nullField = paccessorGen.paGetStringField(operation_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
				//targetNamespace
                nullField = paccessorGen.paGetStringField(targetNamespace_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 9, ((java.lang.String) nullField));
                }
				//partner
                nullField = paccessorGen.paGetStringField(partner_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 10, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 10, ((java.lang.String) nullField));
                }
				//faultMessage
                nullField = paccessorGen.paGetStringField(faultMessage_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 11, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 11, ((java.lang.String) nullField));
                }
				//state
                nullField = paccessorGen.paGetStringField(state_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 12, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 12, ((java.lang.String) nullField));
                }
				//faultName
                nullField = paccessorGen.paGetStringField(faultName_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 13, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 13, ((java.lang.String) nullField));
                }
				//portType
                nullField = paccessorGen.paGetStringField(portType_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 14, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 14, ((java.lang.String) nullField));
                }
				//refPartner
                nullField = paccessorGen.paGetStringField(refPartner_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 15, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 15, ((java.lang.String) nullField));
                }
				//statementNb
				adapter.setInt(pstmt, 16, paccessorGen.paGetIntField(statementNb_NAME));
				//date_
				adapter.setLong(pstmt, 17, paccessorGen.paGetLongField(date_NAME));
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String deletebpelMessage7Table(
    	Connection connection,
    	Object conn,
    	PAccessorGen paccessorGen,
    	MessageBeanAccessor paccessor,
    	PName pname) throws PException, SQLException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            pstmt = connection.prepareStatement(res = "DELETE FROM bpelMessage WHERE id = ?");
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 1, ((java.lang.Integer) nullField));
                }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }

	/**
	 * Is the RdbAdapter to use for managing the various relational database.
	 * This field must not be null after the mapping of the class
	 */
	public RdbAdapter adapter;
	/**
	 * Is the select query to use in the Binding.read method
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The read query is built once time
	 * when the class is mapped.
	 */
	public String readQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String prefetchedExtentQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is NOT activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String extentQuery = null;

	/**
	 * Building of the structure defining the FROM clause with
	 *  JoinedTable instances
	 */
	private JoinedTable getJoinedTable() {
		JoinedTable jt = new JoinedTable("bpelMessage");
		JoinedTable.Join jtj = null;
        return jt;
	}

    public String[] getPrefetchSelectCols() {
		return new String[]{"bpelMessage.BPIEntityBean_id", "bpelMessage.date_", "bpelMessage.fault", "bpelMessage.faultMessage", "bpelMessage.faultName", "bpelMessage.id", "bpelMessage.namespace", "bpelMessage.operation", "bpelMessage.partner", "bpelMessage.portType", "bpelMessage.processName", "bpelMessage.refPartner", "bpelMessage.state", "bpelMessage.statementNb", "bpelMessage.targetNamespace", "bpelMessage.type", "bpelMessage.value"};
	}

	public String getPrefetchWhereClause() {
		return null;
	}

	public java.util.Properties getCNId2Column() {
		java.util.Properties p = new java.util.Properties();
		p.setProperty("", "bpelMessage.id");
		return p;
	}

	public List getPrefetchTables() {
		return Collections.singletonList(getJoinedTable());
	}

	/**
	 * Compute the read query with the help of the RdbAdapter
	 */
	public void computeReadQuery() {
		readQuery = adapter.getQuery("bpelMessage.BPIEntityBean_id, bpelMessage.date_, bpelMessage.fault, bpelMessage.faultMessage, bpelMessage.faultName, bpelMessage.id, bpelMessage.namespace, bpelMessage.operation, bpelMessage.partner, bpelMessage.portType, bpelMessage.processName, bpelMessage.refPartner, bpelMessage.state, bpelMessage.statementNb, bpelMessage.targetNamespace, bpelMessage.type, bpelMessage.value",
			Collections.singletonList(getJoinedTable()),
			"bpelMessage.id = ?",
			false, false);
	}

	/**
	 * Compute the extent query (without prefetch) with the help of the
	 * RdbAdapter
	 */
	public void computeExtentQuery() throws ExpressionException {
		if (extentQuery == null) {
			extentQuery = adapter.getQuery("bpelMessage.id",
				Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}
	}

	/**
	 * Compute the extent query (with prefetch) with the help of the RdbAdapter
	 */
	public void computePrefetchedExtentQuery() {
		if (prefetchedExtentQuery == null) {
			prefetchedExtentQuery = adapter.getQuery("bpelMessage.BPIEntityBean_id"+ adapter.getColumnAliasExpr("bPI_id") + ", bpelMessage.date_"+ adapter.getColumnAliasExpr("date") + ", bpelMessage.fault"+ adapter.getColumnAliasExpr("fault") + ", bpelMessage.faultMessage"+ adapter.getColumnAliasExpr("faultMessage") + ", bpelMessage.faultName"+ adapter.getColumnAliasExpr("faultName") + ", bpelMessage.id"+ adapter.getColumnAliasExpr("id") + ", bpelMessage.namespace"+ adapter.getColumnAliasExpr("namespace") + ", bpelMessage.operation"+ adapter.getColumnAliasExpr("operation") + ", bpelMessage.partner"+ adapter.getColumnAliasExpr("partner") + ", bpelMessage.portType"+ adapter.getColumnAliasExpr("portType") + ", bpelMessage.processName"+ adapter.getColumnAliasExpr("processName") + ", bpelMessage.refPartner"+ adapter.getColumnAliasExpr("refPartner") + ", bpelMessage.state"+ adapter.getColumnAliasExpr("state") + ", bpelMessage.statementNb"+ adapter.getColumnAliasExpr("statementNb") + ", bpelMessage.targetNamespace"+ adapter.getColumnAliasExpr("targetNamespace") + ", bpelMessage.type"+ adapter.getColumnAliasExpr("type") + ", bpelMessage.value"+ adapter.getColumnAliasExpr("value") + ", bpelMessage.id" + adapter.getColumnAliasExpr("pk0") + "",
			Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}

	}


	public PName decode(ResultSet rs, int startindex, Object png) throws IOException, SQLException, PException {
        return classPNC.decodeOint(adapter.getOint(rs, startindex, null));
	}



	
	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
}




class MessageBeanResultIterator implements PNameIterator {
    ResultSet rs;
	private PBinder binder;
	private MessageBeanMapping pclassMapping;
    private boolean hasNext;
    private boolean isForw;
	private PreparedStatement ps;
	private Logger logger;

    private boolean prefetching;
    
    private PPrefetchTupleCollection tc = null;
    private int pnameIndex;

	MessageBeanResultIterator(Object txctx,
		PreparedStatement ps,
		ResultSet r,
		PBinder binder,
		boolean prefetching,
		boolean withSubType,
		Logger l,
		int pi,
		MessageBeanMapping pcm) throws SQLException, MedorException {

		this.ps = ps;
		this.binder = binder;
		this.prefetching = prefetching;
        rs = r;
        logger = l;
        pclassMapping = pcm;
        if (!prefetching) {
        	pnameIndex = 1;
        } else {
        	pnameIndex = pi;
        	tc = new RdbTupleCollection(txctx, r, ps, pnameIndex, logger, pclassMapping, withSubType, this);
        }
        hasNext = (!prefetching) ? rs.next() : tc.next();
        isForw = true;
    }

	public void finalize() throws Throwable {
		if (rs != null)
			rs.close();
		if (ps != null)
			ps.close();
		super.finalize();
	}

    public void close() {
        if (!prefetching) {
            if (rs!= null) {
                try {
                    rs.close();
                } catch(Exception e) {
                }
                rs = null;
            }
            hasNext = false;
            isForw = false;
        }
    }

    public boolean hasNext() {
    	boolean res;
        if (isForw) {
            res = hasNext;
        } else {
            isForw = true;
            try {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                res = hasNext;
            } catch(Exception e) {
                e.printStackTrace();
                res = false;
            }
        }
        if (res == false) {
        	try {
				if (rs != null) {
					rs.close();
					rs = null;
				}
				if (ps != null) {
					ps.close();
					ps = null;
				}
			} catch (SQLException e) {
				rs = null;
				ps = null;
			}
        }
        return res;
    }

    public void remove() throws UnsupportedOperationException, IllegalStateException {
        throw new UnsupportedOperationException();
    }

    public Object next() throws NoSuchElementException {
        try {
            if (!isForw) {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                isForw = true;
            }
            if (!hasNext) {
                throw new NoSuchElementException();
            }
            isForw = false;
            if (prefetching) {
            	tc.getPrefetchBuffer().addPrefetchTuple();
            }
            return pclassMapping.decode(rs, pnameIndex, null);
        } catch (NoSuchElementException e) {
        	throw e;
        } catch (Exception e) {
        	if (logger != null) {
        		logger.log(BasicLevel.ERROR, "Impossible to fetch the next element", e);
        	}
            throw new NoSuchElementException(
                    "Impossible to decode to an ID PName: " + e.getMessage());
        }
    }
}
